from Products.CollectiveGTD.config import PROJECTNAME, ALLTYPES, ALLSKINS
from Products.CMFCore.utils import getToolByName

from StringIO import StringIO
from Products.CMFPlone.migrations import v3_0

def install(self):
    out = StringIO()
    
    # We install our product by running a GS profile.  We use the old-style Install.py module 
    # so that our product works w/ the Quick Installer in Plone 2.5.x
    print >> out, "Installing collectivegtd"
    setup_tool = getToolByName(self, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile('profile-Products.CollectiveGTD:default')
    print >> out, "Installed types and added to portal_factory via portal_setup"

    catalog = getToolByName(self, 'portal_catalog')
    catalog.manage_reindexIndex(ids=['ActionItemStartDate','ActionItemDueDate',
        'ActionItemFlag','ActionItemComplete','ActionItemTags',])
    print >> out, "Reindex custom indexes to get around bug: https://bugs.launchpad.net/zope-cmf/+bug/161682"
    
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    
    # Add collectivegtd types to types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f not in typesNotListed:
            typesNotListed.append(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Added collectivegtd types to metaTypesNotToList"
    
    # need to do this to add the roles to PAS 
    # and supplement genericsetup until 3.0 comes forth
    uf = getToolByName(self, 'acl_users')
    rmanager = uf.portal_role_manager
    roles = ('NavigationManager',)
    existing = rmanager.listRoleIds()
    for role in roles:
        if role not in existing:
            rmanager.addRole(role)
    print >> out, "Added new role NavigationManager"
    
    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue()
    

def uninstall(self):
    out = StringIO()

    portal_factory = getToolByName(self,'portal_factory')
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    types_tool = getToolByName(self, 'portal_types')
    
    # remove collectivegtd types as a factory type
    factory_types = portal_factory.getFactoryTypes().keys()
    for t in ALLTYPES:
        if t in factory_types:
            factory_types.remove(t)
    portal_factory.manage_setPortalFactoryTypes(listOfTypeIds=factory_types)
    print >> out, "Removed collectivegtd types from portal_factory tool"
    
    # remove collectivegtd types from the portal's list of types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f in typesNotListed:
            typesNotListed.remove(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Removed collectivegtd types from metaTypesNotToList"
    
    # Remove skin directory from skin selections
    skinstool = getToolByName(self, 'portal_skins')
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName)
        path = [i.strip() for i in  path.split(',')]
        for strSkinDir in ALLTYPES:
            if strSkinDir in path:
                path.remove(strSkinDir)
        path = ','.join(path)
        skinstool.addSkinSelection(skinName, path)

    print >> out, "Removed collectivegtd skin layers from all skin selections"
    
    print >> out, "\nSuccessfully uninstalled %s." % PROJECTNAME
    return out.getvalue()
