__author__ = """Brian Gershon <briang@webcollective.coop>"""
__docformat__ = 'plaintext'

from zope.interface import implements, alsoProvides
from zope.component import getUtility
from Products import Five
from Products.CMFCore.utils import getToolByName
from Products.validation import validation

from zope.formlib import form
from Products.Five.formlib import formbase
from zope.schema.fieldproperty import FieldProperty
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from zope.component import getMultiAdapter

from zExceptions import BadRequest
from AccessControl import Unauthorized

from Products.CollectiveGTD.interfaces import IActionItemListAdd, \
    IActionItemListQuickAddFields, IActionItemListView, IActionItemListUpdate, \
    ICollectiveGTDUtility, IActionItemListManager, IActionItemListAddFields, \
    IActionItemListAdd, IActionItemListImporter

from DateTime import DateTime
from zope.app.form.interfaces import WidgetInputError
from Products.CMFPlone import PloneMessageFactory as _

from zope.schema.vocabulary import SimpleVocabulary
from zope.app.schema.vocabulary import IVocabularyFactory

from Products.CollectiveGTD.modules import datetimelingo
from datetime import datetime, timedelta

from kss.core import kssaction 
from plone.app.kss.plonekssview import PloneKSSView

import elementtree.ElementTree as ET
from StringIO import StringIO

from Products.CollectiveGTD.browser.widgets import CustomDropdownWidget
from Products.CMFPlone.utils import normalizeString

DATE_DISPLAY_FORMAT = 'yyyy/mm/dd'  # this European format required for simple
                                    # HTML table sorting

class ActionItemListManager(object):
    """ See IActionItemListManager
    """
    implements(IActionItemListManager)
    
    def __init__(self, context):
        self.context = context
        self.catalog = getToolByName(context, 'portal_catalog')
        self.workflow_tool = getToolByName(context, 'portal_workflow')
        self.plone_utils = getToolByName(context, 'plone_utils')

    def _findActionItemTags(self, filter_by_tag, catalog_results):
        """ Pulls back the ActionItemTags from the various brain objects,
            converts list to unicode, then compares against unicode filter_by_tag,
            and returns those that match.
        """
        results = []
        if filter_by_tag and filter_by_tag.lower() != "all":
            for r in catalog_results:
                # convert to unicode
                actionitemtags = [t.decode('utf8') for t in r.ActionItemTags]
                if filter_by_tag in actionitemtags:
                    results.append(r)
            return results
        else:
            return catalog_results
        
    def _ActionItemToDict(self, actionitem_brain):
        # FIXME: Add field(s) to catalog to speed things up (instead of getObject())
        actionItem = actionitem_brain.getObject()

        projects = []
        contexts = []
        if actionitem_brain.ActionItemTags:
            projects = [p[8:] for p in actionitem_brain.ActionItemTags if p.startswith('project:')]
            contexts = [p[8:] for p in actionitem_brain.ActionItemTags if p.startswith('context:')]
        # TODO: Eventually map these tags to a hierarchy as well as a descriptive title

        # Convert Zope DateTime to Python datetime, then add a "Display" version
        #   of the date using datetimelingo
        startDateDisplay = actionitem_brain.ActionItemStartDate
        if actionitem_brain.ActionItemStartDate is not None:
            dt = actionitem_brain.ActionItemStartDate
            startDateDisplay = datetimelingo.displayDate(datetime(dt.year(), dt.month(), dt.day()), \
                DATE_DISPLAY_FORMAT)
            # combine both date and displayDate() to maintain html table sorting
            date_formatted = actionitem_brain.ActionItemStartDate.strftime("%Y/%m/%d")
            if startDateDisplay != date_formatted:
                startDateDisplay = date_formatted \
                + " (%s)" % startDateDisplay

        dueDateDisplay = actionitem_brain.ActionItemDueDate
        if actionitem_brain.ActionItemDueDate is not None:
            dt = actionitem_brain.ActionItemDueDate
            dueDateDisplay = datetimelingo.displayDate(datetime(dt.year(), dt.month(), dt.day()), \
                DATE_DISPLAY_FORMAT)
            # combine both date and displayDate() to maintain html table sorting
            date_formatted = actionitem_brain.ActionItemDueDate.strftime("%Y/%m/%d")
            if dueDateDisplay != date_formatted:
                dueDateDisplay = date_formatted \
                + " (%s)" % dueDateDisplay

        # set css_class based on DueDate
        css_class = ""
        #print actionitem_brain.ActionItemDueDate, DateTime(),actionitem_brain.ActionItemDueDate - DateTime()
        if actionitem_brain.ActionItemDueDate is not None:
            if (actionitem_brain.ActionItemDueDate - DateTime()) < 1:
                css_class = "action-due"
            elif (actionitem_brain.ActionItemDueDate - DateTime()) >=1 \
                and (actionitem_brain.ActionItemDueDate - DateTime()) < 2:
                css_class = "action-upcoming"
        
        d = {
            'id': actionitem_brain.id,
            'created': actionitem_brain.created,
            'absolute_url': actionitem_brain.getURL(),
            'title': actionitem_brain.Title, 
            'priority': "(not implemented yet)",
            'flag': actionitem_brain.ActionItemFlag,
            'recurring': "(not implemented yet)",
            'startDate': actionitem_brain.ActionItemStartDate,
            'dueDate': actionitem_brain.ActionItemDueDate,
            'startDateDisplay': startDateDisplay,
            'dueDateDisplay': dueDateDisplay,
            'effort': "(not implemented yet)",
            'complete': actionitem_brain.ActionItemComplete,
            'portal_type': 'ActionItem',
            'description': actionItem.Description(),
            'projects': projects,
            'contexts': contexts,
            'css_class': css_class,
        }
        return d
        
    def _getActionItems(self, sortfieldname, filter_by_tag):
        
        sort_on = None
        sort_order=""
        if sortfieldname is not None:
            if sortfieldname=="ActionItemFlag":
                sort_on = "ActionItemFlag"
                sort_order="reverse"
            elif sortfieldname=="ActionItemComplete":
                sort_on = "ActionItemComplete"
                sort_order="reverse"
            elif sortfieldname=="ActionItemStartDate":
                sort_on = "ActionItemStartDate"
                sort_order=""
            elif sortfieldname=="ActionItemDueDate":
                sort_on = "ActionItemDueDate"
                sort_order=""
        
        if sort_on is None or sort_on == 'ActionItemStartDate' or sort_on == 'ActionItemDueDate':
            # don't sort date fields using sort_on since that isn't pulling back records
            #   that don't have dates in those fields.  Sort below.
            # 
            catalog_results = self.catalog.searchResults(path='/'.join(self.context.getPhysicalPath()), portal_type='ActionItem')
        else:
            catalog_results = self.catalog.searchResults(path='/'.join(self.context.getPhysicalPath()), portal_type='ActionItem', sort_on=sort_on, sort_order=sort_order)

        results = self._findActionItemTags(filter_by_tag, catalog_results)

        # Create list of dictionaries
        listResults = []
        for item in results:
            listResults.append( self._ActionItemToDict(item) )
            
        # Sort here, and not via sort date fields in portal_catalog since that isn't pulling back
        # records that don't have dates in those fields.
        if sort_on == 'ActionItemStartDate':
            listResults.sort(lambda x, y : cmp (y['startDate'], x['startDate']))
        elif sort_on == 'ActionItemDueDate':
            listResults.sort(lambda x, y : cmp (y['dueDate'], x['dueDate']))

        return listResults
    
    def _getUpcomingActions(self, filter_by_tag):
        # filter by tag first
        catalog_results = self.catalog.searchResults(path='/'.join(self.context.getPhysicalPath()), \
            portal_type='ActionItem', ActionItemComplete=False, sort_on="ActionItemFlag")
        results = self._findActionItemTags(filter_by_tag, catalog_results)

        # TODO: sorted by ActionItemFlag first instead of ActionItemDueDate because
        # searchResults() with that parameter don't return values where DueDate
        # field is None
        
        # Create list of dictionaries
        listResults = []
        for item in results:
            listResults.append( self._ActionItemToDict(item) )

        # How do we sort dueDate of None to the end (out of the natural
        # sort order)?
        # Let's create a new dictionary key with dueDate of None set
        # to the far future
        far_future = DateTime.Date(DateTime() + 100)
        for k in listResults:
            if k['dueDate'] is None:
                k['sortDueDate'] = far_future
            else:
                k['sortDueDate'] = k['dueDate']
        
        # strip out any that start in the future
        non_future_actions = []
        for a in listResults:
            # strip off any potential time, then re-convert into a DateTime
            today = DateTime(DateTime.Date(DateTime()))
            startDate = DateTime(DateTime.Date(DateTime(a['startDate'])))
            if a['startDate'] is None:
                # leave that in the list since there is no StartDate
                non_future_actions.append(a)
            else:
                if startDate <= today:
                    # leave that in the list
                    non_future_actions.append(a)
                else:
                    # startDate exists and is in the future, so skip it!
                    pass
        
        flagged_yes = [a for a in non_future_actions if a['flag']]
        flagged_no = [a for a in non_future_actions if not a['flag']]

        # sort flagged items by DueDate
        sort_on = "sortDueDate"
        decorated = [(dict_[sort_on], dict_) for dict_ in flagged_yes]
        decorated.sort()
        sorted_results_flagged_yes = [dict_ for (key, dict_) in decorated]

        # sort non-flagged items by DueDate
        sort_on = "sortDueDate"
        decorated = [(dict_[sort_on], dict_) for dict_ in flagged_no]
        decorated.sort()
        sorted_results_flagged_no = [dict_ for (key, dict_) in decorated]

        # return results, flagged on top
        return sorted_results_flagged_yes + sorted_results_flagged_no
        
    def _getRecentlyAddedActions(self, filter_by_tag):
        catalog_results = self.catalog.searchResults(path='/'.join(self.context.getPhysicalPath()),
            portal_type='ActionItem',
            #sort_on='Date',            # sorting by a DateTime() index doesn't seem to work consistently
            #sort_order='descending',
        )

        results = self._findActionItemTags(filter_by_tag, catalog_results)
        
        listResults = []
        for item in results:
            listResults.append( self._ActionItemToDict(item) )

        listResults.sort(lambda x, y : cmp (y['created'], x['created']))
        return listResults

    def getSortedActions(self, sortview, sortlimit, filter_by_tag):
        # grab results
        results = []
        
        if sortview == "upcoming":
            results = self._getUpcomingActions(filter_by_tag)
        elif sortview == "recently_added":
            results = self._getRecentlyAddedActions(filter_by_tag)
        elif sortview == "all":
            results = self._getActionItems('', filter_by_tag)
        elif sortview == "sort_done":
            results = self._getActionItems("ActionItemComplete", filter_by_tag)
        elif sortview == "sort_flag":
            results = self._getActionItems("ActionItemFlag", filter_by_tag)
        elif sortview == "sort_startdate":
            results = self._getActionItems("ActionItemStartDate", filter_by_tag)
        elif sortview == "sort_duedate":
            results = self._getActionItems("ActionItemDueDate", filter_by_tag)
        else:
            results = []
            
        # how many records should we display?
        final_results = []
        if results:
            if sortlimit == 'all' or sortlimit == '':
                final_results = results
            else:
                sortlimit_number = int(sortlimit)
                final_results = results[:sortlimit_number]
        return final_results
        
    def _generateID(self, actionitem_title):
        """
        Generate a unique ID for a new ActionItem to be added to this
            ActionItemList.
        """
        folder = self.context
        clean_id = self.plone_utils.normalizeString(actionitem_title)

        # Check to see if ID already exists.  If it does, add a number to it
        increment = 1
        non_dup_id = clean_id
        while non_dup_id in [f.id for f in folder.getFolderContents()]:
            non_dup_id = "%s-%s" % (clean_id, increment)
            increment += 1
        clean_id = non_dup_id
        return clean_id
    
    def _sanitizeFormFields(self, form_data):
        """
        Loop through all dictionary entries, and strip out any non-authorized
        HTML characters.
        
        FIXME: _sanitizeFormFields() needs to be implemented.
        """
        pass
        
    def validateFields(self, form_data):
        
        error_dict = {}
        startDate = None
        dueDate = None
        
        # test startDate
        if form_data.has_key('actionitem_startdate') and \
            form_data['actionitem_startdate'] != u'None' and \
            form_data['actionitem_startdate'] is not None:
            startDate = form_data['actionitem_startdate']
            try:
                if startDate and datetimelingo.createDate(startDate):
                    pass
            except ValueError, DateTime.SyntaxError:
                err_msg = u"Invalid date format."
                error_dict['actionitem_startdate'] = err_msg
            
        # test dueDate
        if form_data.has_key('actionitem_duedate') and \
            form_data['actionitem_duedate'] != u'None' and \
            form_data['actionitem_duedate'] is not None:
            dueDate = form_data['actionitem_duedate']
            try:
                if dueDate and datetimelingo.createDate(dueDate):
                    pass
            except ValueError, DateTime.SyntaxError:
                err_msg = u"Invalid date format."
                error_dict['actionitem_duedate'] = err_msg

        try:
            if startDate and dueDate:
                if datetimelingo.createDate(startDate) > datetimelingo.createDate(dueDate):
                    err_msg = u"Start Date should not be later than Due Date."
                    error_dict['actionitem_startdate'] = err_msg
                    error_dict['actionitem_duedate'] = err_msg
        except ValueError:
            # Let's assume an invalid startDate or dueDate was passed
            # and skip the validation.
            pass
                
        return error_dict
        
    def addActionItem(self, form_data):

        self._sanitizeFormFields(form_data)
        
        clean_id = self._generateID(form_data['actionitem_title'])
        clean_title = form_data['actionitem_title']

        # Make sure we have default values in case the field is not passed
        tags = []
        startDate = None
        dueDate = None
        flag = False
        
        if form_data.has_key('actionitem_startdate'):
            if form_data['actionitem_startdate'] is not None:
                try:
                    cd = datetimelingo.createDate(form_data['actionitem_startdate'])
                    # convert Python datetime to Zope DateTime
                    startDate = DateTime(cd.year, cd.month, cd.day)
                except ValueError:
                    startDate = None

        if form_data.has_key('actionitem_duedate'):
            if form_data['actionitem_duedate'] is not None:
                try:
                    cd = datetimelingo.createDate(form_data['actionitem_duedate'])
                    # convert Python datetime to Zope DateTime
                    dueDate = DateTime(cd.year, cd.month, cd.day)
                except ValueError:
                    dueDate = None

        # First, convert Context and Projects into a list of tags
        # TODO: Substitute these in below once actionitem_project and
        #       actionitem_context are modified to allow multiple value
        #       selection:
        #[tags.append("project:%s" % item) for item in form_data['actionitem_project']]
        #[tags.append("context:%s" % item) for item in form_data['actionitem_context']]

        # NOTE: These if's with "is not None" prevent bug in
        #   testActionItemQuickAddForm_RequiredFieldsOnly.
        #   Though the tests pass with or without this clause,
        #   but without it, the real formlib form breaks.
        
        # If a new context or project is added, use that instead of the drop-down value
        if form_data.has_key('actionitem_newproject') and form_data['actionitem_newproject']:
            tags.append("project:%s" % form_data['actionitem_newproject'])
        elif form_data.has_key('actionitem_project') and form_data['actionitem_project'] is not None:
            tags.append("project:%s" % form_data['actionitem_project'])
            
        if form_data.has_key('actionitem_newcontext') and form_data['actionitem_newcontext']:
            tags.append("context:%s" % form_data['actionitem_newcontext'])
        elif form_data.has_key('actionitem_context') and form_data['actionitem_context'] is not None:
            tags.append("context:%s" % form_data['actionitem_context'])

        if form_data.has_key('actionitem_flag'):
            if form_data['actionitem_flag']:
                flag = True
            else:
                flag = False
        
        # Create the new ActionItem
        self.context.invokeFactory('ActionItem', id=clean_id, \
                title=clean_title, startDate=startDate,
                dueDate=dueDate, tags=tags, flag=flag)
    
class ActionItemListView(Five.BrowserView):
    """ See IActionItemListView
    """
    implements(IActionItemListView)
    
    def __init__(self, context, request):
        Five.BrowserView.__init__(self, context, request)
        self.listmanager = IActionItemListManager(context)
        
    def getSortedActions(self, sortview, sortlimit, filter_by_tag):
        # find proper tag based on normalizeString version
        #  e.g. project:home-123 should convert to project:Home 123
        new_filter_by_tag = ''
        for tag in self.getAllProjectsAndContexts():
            if filter_by_tag == tag[0]:
                if tag[0].startswith("context:"):
                    new_filter_by_tag = "context:%s" % tag[1]
                elif tag[0].startswith("project:"):
                    new_filter_by_tag = "project:%s" % tag[1]

        return self.listmanager.getSortedActions(sortview, sortlimit, new_filter_by_tag)

    def getAllProjectsAndContexts(self):
        """ Return sorted list of all Contexts & Projects with an title/id tuple.
            Group by context, then by project.
            
            We need to change thd id value to use "project:" and "context:" PLUS
            the unicode string.  We stripped unicode out of id because formlib
            requires id to be ascii.  But our form wants the actual unicode
            version.
        """
        # getContexts() and getProjects() already sort alphabetically by Title
        utility = getUtility(ICollectiveGTDUtility)
        
        results = []

        contexts = utility.getContexts(self.context)
        results.extend(contexts)
        
        projects = utility.getProjects(self.context)
        results.extend(projects)

        return results

    def getTagID(self, tag_title):
        """ Return an ID given a tag Title.
            e.g. Used by actions_view for adding links to Contexts and Projects.
        """
        all_tags = self.getAllProjectsAndContexts()
        for t in all_tags:
            id, title = t
            if tag_title.decode("utf8") == title:
                return id
        return '(unknown tag)'

    def getAllActiveActions(self, outputFormat="xml"):
        """Returns all active ActionItems in outputFormat.
        
           Also returns list of Projects and Contexts.
        """
        response = self.request.response

        if outputFormat == "xml":

            # build a tree structure
            root = ET.Element("collectivegtd")
            root.set("language","en")

            actions = ET.SubElement(root, "actionitems")
            
            all_actions = self.getSortedActions(sortview="upcoming", \
                sortlimit="all", filter_by_tag="all")
            for a in all_actions:
                el = ET.SubElement(actions, "actionitem")
                el.set("id", a['id'])
                el.set("title", a['title'])

                if hasattr(a['startDate'], 'strftime'):
                    el.set("startdate", a['startDate'].strftime("%m/%d/%Y"))
                else:
                    el.set("startdate", '')
                    
                if hasattr(a['dueDate'], 'strftime'):
                    el.set("duedate", a['dueDate'].strftime("%m/%d/%Y"))
                else:
                    el.set("duedate", '')
                    
                el.set("flag", str(a['flag']))
                
                proj = ET.SubElement(el,"project_tags")
                projects = a['projects']
                if projects:
                    for p in projects:
                        project_element = ET.SubElement(proj, "tag")
                        project_element.set("title",p)

                context = ET.SubElement(el, "context_tags")
                contexts = a['contexts']
                if contexts:
                    for c in contexts:
                        context_element = ET.SubElement(context, "tag")
                        context_element.set("title", c)


            utility = getUtility(ICollectiveGTDUtility)

            all_contexts = ET.SubElement(root, "allcontexts")
            for (friendly_tag, full_tag) in utility.getContexts(self.context):
                context = ET.SubElement(all_contexts, "context")
                context.set("title", friendly_tag)

            all_projects = ET.SubElement(root, "allprojects")
            for (friendly_tag, full_tag) in utility.getProjects(self.context):
                proj = ET.SubElement(all_projects, "project")
                proj.set("title", friendly_tag)

            # wrap it in an ElementTree instance, and save as XML
            out = StringIO()
            tree = ET.ElementTree(root)
            tree.write(out)

            response.setHeader('Content-Type', 'text/xml')
            return out.getvalue()
        else:
            return None
    
class ActionItemListQuickAdd(formbase.Form):
    """See IActionItemListAdd
    """
    implements(IActionItemListAdd)

    form_fields = form.Fields(IActionItemListQuickAddFields)
    form_fields['actionitem_context'].custom_widget = CustomDropdownWidget
    form_fields['actionitem_project'].custom_widget = CustomDropdownWidget
    
    template = ViewPageTemplateFile('actionitemlist_quickadd.pt')

    status = errors = None
    prefix = 'form'
    
    label = u"ActionItemList Quick Add Form"
    description = u'This page allows you to quickly add new ActionItems to this ActionItemList.'
    
    actions = form.Actions(
        form.Action('Add', success='handle_add_action',
                        failure='handle_add_action_failure',
                        validator='handle_add_action_validator'),
    )
    
    def __init__( self, context, request ):
        formbase.Form.__init__(self, context, request)
        self.context = context
        self.request = request
        self.plone_utils = getToolByName(self.context, 'plone_utils')
        self.listmanager = IActionItemListManager(context)

    def setUpWidgets(self, ignore_request=False):
        # data to initialize the form with
        data = {}
        
        self.widgets = form.setUpWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            data=data,
            ignore_request=ignore_request
            )

    #@form.action("Add QuickAction", failure='handle_edit_action_failure')
    def handle_add_action(self, action, data):
        """ Add a new ActionItem to this ActionItemList """

        try:
            self.listmanager.addActionItem(form_data=data)
        except BadRequest,e:
            self.status = "Update FAILED due to BadRequest '%s'" % e
            self.plone_utils.addPortalMessage(_(u"Update FAILED due to BadRequest '%s'" % e))
            return
        except Unauthorized, e:
            # what if user doesn't have permissions?
            self.status = "Update FAILED due to Unauthorized '%s'" % e
            self.plone_utils.addPortalMessage(_(u"Update FAILED due to Unauthorized '%s'" % e))
            return
        except:
            self.status = 'Update FAILED for an unexpected reason when trying to create...'
            self.plone_utils.addPortalMessage(_(u'Update FAILED for an unexpected reason...'))
            return
        
        self.status = 'Updated successfully.'
        self.plone_utils.addPortalMessage(_(u'Updated successfully.'))

    def handle_add_action_failure(self, action, data, errors):
        self.status = 'There were %d error(s).' % len(errors)
        if len(errors) == 1:
            self.plone_utils.addPortalMessage(_(u'There was %d error.' % len(errors)))
        else:
            self.plone_utils.addPortalMessage(_(u'There were %d errors.' % len(errors)))
        
    def handle_add_action_validator(self, action, data):
        """
        """
        errors = []
        # Handle default formlib checking, such as required fields
        errors.extend( 
                form.getWidgetsData(self.widgets, self.prefix, data) +
                form.checkInvariants(self.form_fields, data)
        )

        # FIXME: Move this validation code into "invariant" functions?
        field_errors = self.listmanager.validateFields(form_data = data)

        if field_errors:
            if field_errors.has_key('actionitem_startdate'):
                err_msg = field_errors['actionitem_startdate']
                self.widgets['actionitem_startdate'].error = err_msg
                # set Widget error notice at top of form
                errors.append(WidgetInputError(
                    field_name='actionitem_startdate',
                    widget_title=u'Start Date',
                    errors=err_msg),
                    )

            if field_errors.has_key('actionitem_duedate'):
                err_msg = field_errors['actionitem_duedate']
                self.widgets['actionitem_duedate'].error = err_msg
                # set Widget error notice at top of form
                errors.append(WidgetInputError(
                    field_name='actionitem_duedate',
                    widget_title=u'Due Date',
                    errors=err_msg),
                    )
                    
        return errors

class ActionItemListAdd(formbase.Form):
    """See IActionItemListAdd
    """
    implements(IActionItemListAdd)

    form_fields = form.Fields(IActionItemListAddFields)
    form_fields['actionitem_context'].custom_widget = CustomDropdownWidget
    form_fields['actionitem_project'].custom_widget = CustomDropdownWidget

    template = ViewPageTemplateFile('actionitemlist_add.pt')

    status = errors = None
    prefix = 'form'

    label = u"ActionItemList Add Form"
    description = u'This page allows you to add new ActionItems to this ActionItemList.'

    actions = form.Actions(
        form.Action('Add', success='handle_add_action',
                        failure='handle_add_action_failure',
                        validator='handle_add_action_validator'),
    )

    def __init__( self, context, request ):
        formbase.Form.__init__(self, context, request)
        self.context = context
        self.request = request
        self.plone_utils = getToolByName(self.context, 'plone_utils')
        self.listmanager = IActionItemListManager(context)

    def setUpWidgets(self, ignore_request=False):
        # data to initialize the form with
        data = {}

        self.widgets = form.setUpWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            data=data,
            ignore_request=ignore_request
            )

    #@form.action("Add QuickAction", failure='handle_edit_action_failure')
    def handle_add_action(self, action, data):
        """ Add a new ActionItem to this ActionItemList """

        try:
            self.listmanager.addActionItem(form_data=data)
        except BadRequest,e:
            self.status = "Update FAILED due to BadRequest '%s'" % e
            self.plone_utils.addPortalMessage(_(u"Update FAILED due to BadRequest '%s'" % e))
            return
        except Unauthorized, e:
            # what if user doesn't have permissions?
            self.status = "Update FAILED due to Unauthorized '%s'" % e
            self.plone_utils.addPortalMessage(_(u"Update FAILED due to Unauthorized '%s'" % e))
            return
        except:
            self.status = 'Update FAILED for an unexpected reason when trying to create...'
            self.plone_utils.addPortalMessage(_(u'Update FAILED for an unexpected reason...'))
            return

        self.status = 'Updated successfully.'
        self.plone_utils.addPortalMessage(_(u'Updated successfully.'))

    def handle_add_action_failure(self, action, data, errors):
        self.status = 'There were %d error(s).' % len(errors)
        if len(errors) == 1:
            self.plone_utils.addPortalMessage(_(u'There was %d error.' % len(errors)))
        else:
            self.plone_utils.addPortalMessage(_(u'There were %d errors.' % len(errors)))

    def handle_add_action_validator(self, action, data):
        """
        """
        errors = []
        # Handle default formlib checking, such as required fields
        errors.extend( 
                form.getWidgetsData(self.widgets, self.prefix, data) +
                form.checkInvariants(self.form_fields, data)
        )

        # FIXME: Move this validation code into "invariant" functions?
        field_errors = self.listmanager.validateFields(form_data = data)

        if field_errors:
            if field_errors.has_key('actionitem_startdate'):
                err_msg = field_errors['actionitem_startdate']
                self.widgets['actionitem_startdate'].error = err_msg
                # set Widget error notice at top of form
                errors.append(WidgetInputError(
                    field_name='actionitem_startdate',
                    widget_title=u'Start Date',
                    errors=err_msg),
                    )

            if field_errors.has_key('actionitem_duedate'):
                err_msg = field_errors['actionitem_duedate']
                self.widgets['actionitem_duedate'].error = err_msg
                # set Widget error notice at top of form
                errors.append(WidgetInputError(
                    field_name='actionitem_duedate',
                    widget_title=u'Due Date',
                    errors=err_msg),
                    )
        return errors

class ActionItemListImporter(Five.BrowserView):
    """ See IActionItemListImporter
    """
    implements(IActionItemListImporter)
    
    def __init__(self, context, request):
        Five.BrowserView.__init__(self, context, request)
        
    def __call__(self):
        """ Process posted data and create an ActionItem """
        form = getMultiAdapter((self.context, self.request), name='actionitemlist_add')

        number_of_folder_items = len(self.context.objectIds())

        form.update()

        # poor man's "check to see if anything was added"
        if (number_of_folder_items + 1) != len(self.context.objectIds()):
            self.request.RESPONSE.setStatus("InternalError", \
                reason="ActionItem not added. Invalid date field?")   # 500 InternalError
            return "ActionItem not added"
            
        self.request.RESPONSE.setStatus(200)
        return "done"
    
class ActionItemListUpdate(Five.BrowserView):
    """ See IActionItemListUpdate
    """
    implements(IActionItemListUpdate)
    
    def __init__(self, context, request):
        Five.BrowserView.__init__(self, context, request)
        self.catalog = getToolByName(self.context, 'portal_catalog')
        self.plone_utils = getToolByName(self.context, 'plone_utils')

    def __call__(self, *args, **kwargs):
        request = self.context.REQUEST
        actionitems = self.request.get('actionitems', '')
        
        if actionitems:
            for ai in actionitems:
                results = self.catalog.searchResults(path='/'.join(self.context.getPhysicalPath()),
                    portal_type='ActionItem',
                    id=ai['id'])
                    
                # strip out any unexpected keys by just adding valid
                # ones to the dictionary.  del ai[k] didn't work since
                # python thinks ai is an instance and not a dict
                clean_ai = {}
                for k in ai.keys():
                    if k in ['id', 'complete', 'flag']:
                        clean_ai[k] = ai[k]

                if results:
                    obj = results[0].getObject()
                    if clean_ai.has_key('flag'):
                        obj.setFlag(True)
                    else:
                        obj.setFlag(False)

                    if clean_ai.has_key('complete'):
                        obj.setComplete(True)
                    else:
                        obj.setComplete(False)
                        
                    obj.reindexObject()

        self.plone_utils.addPortalMessage(_(u'ActionItem(s) updated.'))
        try:
            self.request.response.redirect("%s" % (self.request.environ['HTTP_REFERER']))
        except:
            # unit tests don't have the environ dictionary
            self.request.response.redirect("%s" % (self.context.absolute_url()))

class ActionItemListKSS(PloneKSSView):
    """ Update individual fields using KSS.
    """
    #def __init__(self, context, request):
    #    self.catalog = getToolByName(self.context, 'portal_catalog')

    def _truncateString(self, mystring, desired_length=25):
        unicode_string = mystring.decode('utf8')
        if len(unicode_string) <= desired_length:
            return unicode_string
        else:
            return "%s..." % (unicode_string[:desired_length])

    # TODO: Move these _date* helper functions into an adapter
    #       usable by all views
    def _lingodateEncode(self, datestring):
        """ Helper function to set a DateTime field using datetimelingo
            return Zope DateTime """
        cd = datetimelingo.createDate(datestring)
        return DateTime(cd.year, cd.month, cd.day)

    def _lingodateDisplay(self, dateField):
        """ Helper function to display a DateTime using datetimelingo """
        dt = dateField
        friendlyDate = datetimelingo.displayDate( \
            datetime(dt.year(), dt.month(), dt.day()), \
            DATE_DISPLAY_FORMAT)
        # combine both date and displayDate() to maintain html table sorting
        date_formatted = dt.strftime("%Y/%m/%d")
        
        # always display date, but append lingo friendly date if it exists
        # e.g. if date is "today", display "3/11/08 (today)"
        if friendlyDate != date_formatted:
            return date_formatted + " (%s)" % friendlyDate
        else:
            return date_formatted
        
    def _dateDisplayCSS(self, datefield):
        """ Helper function to grab proper CSS to color code date entry """
        css_class = ""
        if datefield is not None:
            if (datefield - DateTime()) < 1:
                css_class = "action-due"
            elif (datefield - DateTime()) >=1 \
                and (datefield - DateTime()) < 2:
                css_class = "action-upcoming"
        return css_class
        
    def _updateDateField(self, obj, datetimeAccessor, datetimeMutator, datestring):
        """ Update a DateTime field with datestring value.
            Pass in a DateTime field accessor/mutator.
            Raise error if problem e.g. ValueError if invalid datestring.
            Return date ready for display.
        """
        
        dateDisplay = datestring
        
        # if blank datestring, clear date field
        if not datestring:
            datetimeMutator(None)
            dateDisplay="add date"
        else:
            # pass date through datetimelingo, and convert to Zope DateTime
            dt = self._lingodateEncode(datestring)
            datetimeMutator(dt)

            # update page and display portal info message about change
            obj.reindexObject()
            dtstring = datetimeAccessor()
            dateDisplay = self._lingodateDisplay(dtstring)
        return dateDisplay

    @kssaction
    def update_dueDate(self, actionItemID, dueDate):
        core = self.getCommandSet('core')
        plone = self.getCommandSet('plone')
        self.catalog = getToolByName(self.context, 'portal_catalog')
        #request = self.context.REQUEST

        newDate = dueDate
        if actionItemID:
            # TODO: Not sure why this doesn't work, so using non-path one below
            #itemlist_path = '/'.join(self.context.getPhysicalPath())
            #results = self.catalog.searchResults(path=itemlist_path, portal_type='ActionItem', id=actionItemID)
            results = self.catalog.searchResults(portal_type='ActionItem', id=actionItemID)

            if len(results) == 1:
                try:
                    obj = results[0].getObject()
                    dateDisplay = self._updateDateField(obj, obj.ActionItemDueDate, obj.setDueDate, newDate)

                    # update two tags - ID with "main_duedate" prefix
                    #  and ID with "portlet_duedate" prefix so that updating
                    #  one will show an update in the other too.
                    
                    css_class = self._dateDisplayCSS(obj.dueDate)
                    
                    prefix="main_duedate"
                    css_id = '%s-%s' % (prefix, actionItemID)
                    css_tag = """<span id="%s" class="%s label_inlineedit">%s</span>""" % \
                                (css_id, css_class, dateDisplay)
                    core.replaceHTML("#%s" % css_id, css_tag)

                    prefix="portlet_duedate"
                    css_id = '%s-%s' % (prefix, actionItemID)
                    css_tag = """<span id="%s" class="%s label_inlineedit">%s</span>""" % \
                                (css_id, css_class, dateDisplay)
                    core.replaceHTML("#%s" % css_id, css_tag)
                    
                    plone.issuePortalMessage("Changed Due Date for '%s' to '%s'." % \
                                        (self._truncateString(obj.Title()), newDate), msgtype='info')

                except ValueError:
                    plone.issuePortalMessage("Error updating '%s' content with '%s'." % \
                                        (self._truncateString(obj.Title()), newDate), msgtype='error')
                    return
                
            else:
                plone.issuePortalMessage("Didn't find '%s' content." % \
                                    (actionItemID), msgtype='info')

    @kssaction
    def update_startDate(self, actionItemID, startDate):
        core = self.getCommandSet('core')
        plone = self.getCommandSet('plone')
        self.catalog = getToolByName(self.context, 'portal_catalog')
        #request = self.context.REQUEST

        newDate = startDate
        if actionItemID:
            # TODO: Not sure why this doesn't work, so using non-path one below
            #itemlist_path = '/'.join(self.context.getPhysicalPath())
            #results = self.catalog.searchResults(path=itemlist_path, portal_type='ActionItem', id=actionItemID)
            results = self.catalog.searchResults(portal_type='ActionItem', id=actionItemID)

            if len(results) == 1:
                try:
                    obj = results[0].getObject()
                    dateDisplay = self._updateDateField(obj, obj.ActionItemStartDate, obj.setStartDate, newDate)

                    # update two tags - ID with "main_duedate" prefix
                    #  and ID with "portlet_duedate" prefix so that updating
                    #  one will show an update in the other too.

                    css_class = self._dateDisplayCSS(obj.startDate)

                    prefix="main_startdate"
                    css_id = '%s-%s' % (prefix, actionItemID)
                    css_tag = """<span id="%s" class="%s label_inlineedit">%s</span>""" % \
                                (css_id, css_class, dateDisplay)
                    core.replaceHTML("#%s" % css_id, css_tag)

                    plone.issuePortalMessage("Changed Start Date for '%s' to '%s'." % \
                                        (self._truncateString(obj.Title()), newDate), msgtype='info')

                except ValueError:
                    plone.issuePortalMessage("Error updating '%s' content with '%s'." % \
                                        (self._truncateString(obj.Title()), newDate), msgtype='error')
                    return

            else:
                plone.issuePortalMessage("Didn't find '%s' content." % \
                                    (actionItemID), msgtype='info')

    @kssaction
    def update_complete(self, actionItemID, newvalue):
        core = self.getCommandSet('core')
        plone = self.getCommandSet('plone')
        self.catalog = getToolByName(self.context, 'portal_catalog')
        
        if actionItemID:
            results = self.catalog.searchResults(portal_type='ActionItem', id=actionItemID)
        
            if len(results) == 1:
                obj = results[0].getObject()
        
                if newvalue == "on":
                    toggle = "on"
                    obj.setComplete(True)
                else:
                    toggle = "off"
                    obj.setComplete(False)
                    
                obj.reindexObject()

                css_class = "complete-%s" % toggle
            
                prefix="main_complete"
                css_id = '%s-%s' % (prefix, actionItemID)
                css_tag = """<a id="%s" class="%s kssattr-%s kssattr-actionItemID-%s" />""" % \
                            (css_id, css_class, css_class, actionItemID)

                core.replaceHTML("#%s" % css_id, css_tag)

                if toggle == 'on':
                    plone.issuePortalMessage("'%s' has been completed." % \
                                    (self._truncateString(obj.Title())), msgtype='info')
                else:
                    plone.issuePortalMessage("'%s' has not been completed." % \
                                    (self._truncateString(obj.Title())), msgtype='info')

    @kssaction
    def update_flag(self, actionItemID, newvalue):
        core = self.getCommandSet('core')
        plone = self.getCommandSet('plone')
        self.catalog = getToolByName(self.context, 'portal_catalog')
        
        if actionItemID:
            results = self.catalog.searchResults(portal_type='ActionItem', id=actionItemID)
        
            if len(results) == 1:
                obj = results[0].getObject()
        
                if newvalue == "on":
                    toggle = "on"
                    obj.setFlag(True)
                else:
                    toggle = "off"
                    obj.setFlag(False)
                    
                obj.reindexObject()
                
                css_class = "flag-%s" % toggle
                
                prefix="main_flag"
                css_id = '%s-%s' % (prefix, actionItemID)
                css_tag = """<a id="%s" class="%s kssattr-%s kssattr-actionItemID-%s" />""" % \
                            (css_id, css_class, css_class, actionItemID)
                core.replaceHTML("#%s" % css_id, css_tag)
                
                prefix="portlet_flag"
                css_id = '%s-%s' % (prefix, actionItemID)
                css_tag = """<a id="%s" class="%s kssattr-%s kssattr-actionItemID-%s" />""" % \
                            (css_id, css_class, css_class, actionItemID)
                core.replaceHTML("#%s" % css_id, css_tag)
                
                if toggle == 'on':
                    plone.issuePortalMessage("'%s' has been flagged." % \
                                    (self._truncateString(obj.Title())), msgtype='info')
                else:
                    plone.issuePortalMessage("'%s' has been unflagged." % \
                                    (self._truncateString(obj.Title())), msgtype='info')

class CollectiveGTDUtility(object):
    """ A utility that returns site-wide information such as
        all Projects and Contexts
    """
    implements(ICollectiveGTDUtility)
    
    def getProjects(self, context):
        """ Return all Projects with an id/title tuple """
        catalog = getToolByName(context, 'portal_catalog')        

        # grab all tags, then filter into Projects
        raw_tags = catalog.uniqueValuesFor('ActionItemTags')
        
        results = []    #[(u'Please choose a project...', u'none')]
        for item_id, item_name in [(p,p[8:]) for p in raw_tags if p.startswith('project:')]:
            # append id, title
            results.append(("project:%s" % normalizeString(item_id,context)[8:], item_name.decode('utf8'),))
        # sort alphabetically by Name
        results.sort(lambda x, y : cmp (x[1], y[1]))
        return results
        
    def getContexts(self, context):
        """ Return all Contexts with an id/title tuple """
        catalog = getToolByName(context, 'portal_catalog')        

        # grab all tags, then filter into Contexts
        raw_tags = catalog.uniqueValuesFor('ActionItemTags')
        
        results = []    #[(u'Please choose a context...', u'none')]
        for item_id, item_name in [(p,p[8:]) for p in raw_tags if p.startswith('context:')]:
            # append id, title
            results.append(("context:%s" % normalizeString(item_id,context)[8:], item_name.decode('utf8'),))
        # sort alphabetically by Name
        results.sort(lambda x, y : cmp (x[1], y[1]))
        return results

def getProjectVocabulary(context):
    """ Return a SimpleVocabulary of Project tags
        for formlib (using ICollectiveGTDUtility)
    """
    utility = getUtility(ICollectiveGTDUtility)
    return SimpleVocabulary.fromItems(utility.getProjects(context))

def getContextVocabulary(context):
    """ Return a SimpleVocabulary of Context tags
        for formlib (using ICollectiveGTDUtility)
    """
    utility = getUtility(ICollectiveGTDUtility)
    return SimpleVocabulary.fromItems(utility.getContexts(context))

alsoProvides(getProjectVocabulary, IVocabularyFactory)
alsoProvides(getContextVocabulary, IVocabularyFactory)

