from zope import schema
from zope.component import getMultiAdapter
from zope.formlib import form
from zope.interface import implements

from plone.app.portlets.portlets import base
from plone.memoize import ram
from plone.memoize.compress import xhtml_compress
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.cache import render_cachekey

from Acquisition import aq_inner
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone import PloneMessageFactory as _

class IActiveActionItemsPortlet(IPortletDataProvider):

    count = schema.Int(title=_(u'Number of items to display'),
                       description=_(u'How many items to list.'),
                       required=True,
                       default=10)

class Assignment(base.Assignment):
    implements(IActiveActionItemsPortlet)

    def __init__(self, count=5):
        self.count = count

    @property
    def title(self):
        return _(u"Active ActionItems")

def _render_cachekey(fun, self):
    if self.anonymous:
        raise ram.DontCache()
    return render_cachekey(fun, self)

class Renderer(base.Renderer):
    _template = ViewPageTemplateFile('activeactionitems.pt')

    def __init__(self, *args):
        base.Renderer.__init__(self, *args)

        context = aq_inner(self.context)
        portal_state = getMultiAdapter((context, self.request), name=u'plone_portal_state')
        self.anonymous = portal_state.anonymous()
        self.portal_url = portal_state.portal_url()
        self.typesToShow = portal_state.friendly_types()

        plone_tools = getMultiAdapter((context, self.request), name=u'plone_tools')
        self.catalog = plone_tools.catalog()

        if context.portal_type == 'ActionItemList':
            # FIXME: How do I offically make the portlet only show for specific Interfaces?
            #        since the plone:portlet zcml directive already uses "for" to determine
            #        where the portlet should go.
            self.actionitemlist_view = getMultiAdapter((context, self.request), name='actionitemlist_view')
        else:
            self.actionitemlist_view = None
        
    #@ram.cache(_render_cachekey)
    def render(self):
        return xhtml_compress(self._template())

    @property
    def available(self):
        return not self.anonymous and len(self._data())

    def getActiveActionItems(self):
        return self._data()

    #@memoize
    def _data(self):
        limit = self.data.count
        if self.actionitemlist_view:
            return self.actionitemlist_view.getSortedActions(sortview='upcoming', sortlimit=limit, filter_by_tag='')
        else:
            return []

class AddForm(base.AddForm):
    form_fields = form.Fields(IActiveActionItemsPortlet)
    label = _(u"Add Active ActionItems Portlet")
    description = _(u"This portlet displays active ActionItems.")

    def create(self, data):
        return Assignment(count=data.get('count', 10))

class EditForm(base.EditForm):
    form_fields = form.Fields(IActiveActionItemsPortlet)
    label = _(u"Edit Active ActionItems Portlet")
    description = _(u"This portlet displays active ActionItems.")
