# File: ActionItem.py
#
# Copyright (c) 2007 by Web Collective, Inc.
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

__author__ = """Derek Hoshiko <derekh@webcollective.coop>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from Products.CollectiveGTD.config import *
from Products.CMFCore.permissions import View

## Zope
from zope.interface import implements
from Products.CMFCore.utils import getToolByName

# CollectiveGTD
from Products.CollectiveGTD.interfaces import IActionItem

schema = Schema((
    IntegerField(
        name='priority',
        widget=IntegerWidget(
            label="Priority",
            description="(not implemented yet)",
            i18n_domain="collectivegtd",
        ),
    ),
    BooleanField(
        name='flag',
        accessor='ActionItemFlag',
        widget=BooleanWidget(
            label="Flag",
            description="",
            i18n_domain="collectivegtd",
        ),
    ),
    TextField("notes",
          searchable=1,
          default_output_type="text/html",
          allowable_content_types=("text/plain",
                                   "text/restructured",
                                   "text/html",
                                   "application/msword"),
          widget  = RichWidget(
              label="Notes",
              description="",
              i18n_domain="collectivegtd",
          ),
    ),
    LinesField(
        name='tags',
        accessor='ActionItemTags',
        multiValued=1,
        widget=KeywordWidget(
            label="Tags used for setting Projects and Contexts",
            description="",
            i18n_domain="collectivegtd",            
        ),
    ),
    BooleanField(
        name='recurring',
        widget=BooleanWidget(
            label="Recurring",
            description="(not implemented yet)",
            i18n_domain="collectivegtd",
        ),
    ),
    DateTimeField(
        name='startDate',
        accessor='ActionItemStartDate',
        widget=CalendarWidget(
            label="",
            description="",
            i18n_domain="collectivegtd",
        ),
    ),
    DateTimeField(
        name='dueDate',
        accessor='ActionItemDueDate',
        widget=CalendarWidget(
            label="",
            description="",
            i18n_domain="collectivegtd",
        ),
    ),
    StringField(
        name='effort',
        widget=StringWidget(
            label="Effort",
            description="(not implemented yet)",
            i18n_domain="collectivegtd",
        ),
    ),
    BooleanField(
        name='complete',
        accessor='ActionItemComplete',
        widget=BooleanWidget(
            label="",
            description="",
            i18n_domain="collectivegtd",
        ),
    ),
    StringField(
        name='source',
        widget=StringWidget(
            label="Source",
            description="Source where this ActionItem information came from",
            i18n_domain="collectivegtd",
        ),
    ),
),
)

ActionItem_schema = BaseSchema.copy() + schema.copy()

class ActionItem(BaseContent):
    """ ActionItem is a storage container for action item information """
    security = ClassSecurityInfo()

    implements(IActionItem)

    __implements__ = (getattr(BaseContent,'__implements__',()),)

    _at_rename_after_creation = True

    schema = ActionItem_schema

registerType(ActionItem, PROJECTNAME)
