# iGTDUpload.py
# A simple script to add ActionItems to CollectiveGTD directly from an iGTD CSV 
# file export. Useful for importing new ActionItems.
#
# Note: This script is specifically designed to process iGTD CSV file exports.
# Create or use a different process for other upload scenarios.
# 
# Author: Brian Gershon <briang@webcollective.coop>, Feb 2008

import httplib
from urllib import urlencode
import sys
import csv

if len(sys.argv) != 6:
    print """
DESCRIPTION: A script to import ActionItems read in from an iGTD text export.
             (This is also good sample code useful for writing your own importer.)

PARAMETERS:  '%s' expects 5 parameters:

             server_host (e.g. 'localhost:8080')
             server_post (e.g. '/gtd/list-of-stuff/@@actionitemlist_importer')
                IMPORTANT NOTE: list-of-stuff represents an ActionItemList
                                where you want the import items to be saved
                                (so must be created prior to the import).
             login_user (e.g. admin)
             login_pass (e.g. admin)
             path_to_iGTD_text_export_csv (e.g. ~/Desktop/iGTD_export.csv)
""" % sys.argv[0]
    sys.exit()
else:
    server_host = sys.argv[1]   #'localhost:8080'
    server_post = sys.argv[2]   #'/gtd/list-of-stuff/@@actionitemlist_importer'
    login_user = sys.argv[3]    #'admin'
    login_pass = sys.argv[4]    #'admin'
    path_to_csv = sys.argv[5]  # ~/Desktop/iGTD_export.csv

reader = csv.reader(open(path_to_csv,"U"))
reader.next()  # skip header row
reader.next()  # skip blank row under header
importline = 3

headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}
conn = httplib.HTTPConnection(server_host)

print "IMPORT STARTED"

for row in reader:
    if row and len(row)==7:
        # strip out double-quotes
        for s in row:
            s = s.replace('"','')
        # NOTE: Do not change the order of these fields, they are exported from
        # iGTD this way.
        project_name, task_name, context, priority, start_date, due_date, contact = row

        # TODO:
        #  - Skipping priority since not implemented yet in CollectiveGTD.  Add
        #    this functionality when available.
        # NOTE: 'Contact' field is ignored on import.
        if task_name:
            aiFields = {
                '__ac_name': login_user,
                '__ac_password': login_pass,
                'form.actionitem_title': task_name,
                'form.actionitem_startdate': start_date,
                'form.actionitem_duedate': due_date,
                'form.actionitem_newcontext': context,
                'form.actionitem_newproject': project_name,
                'form.actionitem_flag': '   ',   # 'on' to set this 
                'form.actions.add':'Add'
            }
            ActionItem = urlencode(aiFields)
            conn.request("POST", server_post, ActionItem, headers)
            response = conn.getresponse()
            if response.status != 200:
                print "ERROR in line #%s '%s'." % (importline, task_name), 'ERROR IS:', response.status, response.reason
            else:
                print "OK line #%s." % importline
            conn.close()
    else:
        # if blank line, skip notifying us of an error
        if len(row) > 0:
            print 'ERROR', "line #%s" % importline, "INCORRECT NUMBER OF FIELDS: %s" % row
    importline += 1

print "IMPORT COMPLETED"
