__author__ = """Brian Gershon <briang@webcollective.coop>"""
__docformat__ = 'plaintext'

from zope.interface import Interface
from zope import schema
from plone.theme.interfaces import IDefaultPloneLayer

class IActionItem(Interface):
    """ Marker for an ActionItem """

class IActionItemList(Interface):
    """ Marker for an ActionItemList """

class IActionItemListManager(Interface):
    """
    An adapter handling all methods related to IActionItemList folderish
    content-item.  Used by various Forms and Views.
    """
    def getSortedActions(sortview, sortlimit, filter_by_tag):
        """ 
        Returns action items based on two parameters:
            sortview - id of view or sort
            sortlimit - number of items to return
        """
    def validateFields(form_data):
        """ Return a dictionary of errors, key'd by field name """

    def addActionItem(data):
        """
        Add a new ActionItem to this ActionItemList.
        """

class IActionItemListView(Interface):
    """
    ActionItemList browser view.
    
    Uses IActionItemListManager adapter.
    """
    
    def getSortedActions(sortview, sortlimit, filter_by_tag):
        """ 
        Returns action items based on two parameters:
            sortview - id of view or sort
            sortlimit - number of items to return
        """

    def getAllProjectsAndContexts():
        """ Return sorted list of all Contexts & Projects with an title/id tuple.
            Group by context, then by project
        """
        
    def getTagID(tag_title):
        """ Return an ID given a tag Title.
        
            e.g. Used by actions_view for adding links to Contexts and Projects.
        """
        
    def getAllActiveActions(outputFormat="xml"):
        """ Returns all Active Actions for a given ActionItemList.
            Also returns list of Projects and Contexts.

            outputFormat defaults to "xml"
            
            (future support could include "json" for example)
        """
    
class IActionItemListAdd(Interface):
    """Interface for ActionItemListQuickAdd browser view
    """
    
    def setUpWidgets():
        """ Create widgets and fill with default values
        """
        
    def handle_add_action(action, data):
        """ Create a new NavigationItem in the proper place based on
        navigation_location_top_level.  Giving it a title and summary.
        """

    def handle_add_action_failure(action, data, errors):
        """"""
        
    def handle_add_action_validator(action, data):
        """ Validate fields during an Add action
        
            field data comes in from self.request
            
            (I think data is just for extra data,
                not the field data)
        """

class IActionItemListImporter(Interface):
    """ Handles external scripts posting new Actions
        to an ActionItemList folder
    """
    
    def __call__():
        """ Just call the View directly """

class IActionItemListQuickAddFields(Interface):
    """ Fields in the ActionItemListQuickAdd form """

    actionitem_title = schema.TextLine(
                title = u"Action Name",
                description=u"What is the name of the action you want to add?",
                required = True,
                #default = u""
                )

    actionitem_context = schema.Choice(
                title = u"Context Tag",
                description = u"""Please choose a Context tag. Optional. A context is a physical or virtual location where you would perform this task (e.g. office, phone, staff meeting).""",
                required = False,   # note: false automatically generates a (no value) option
                vocabulary = 'ContextVocabulary',   # Vocabulary Factory
                )

    actionitem_newcontext = schema.TextLine(
                title = u"New Context Tag",
                description=u"If your Context doesn't exist in the above drop-down, type one here.",
                required = False,
                )

    actionitem_project = schema.Choice(
                title = u"Project Tag",
                description = u"""Please choose a Project tag. Optional.""",
                required = False,   # note: false automatically generates a (no value) option
                vocabulary = 'ProjectVocabulary',   # Vocabulary Factory
                )

    actionitem_newproject = schema.TextLine(
                title = u"New Project Tag",
                description=u"If your Project doesn't exist in the above drop-down, type one here.",
                required = False,
                )

    actionitem_startdate = schema.TextLine(
                title = u"Start Date",
                description=u"When would you like to see this action become active? Date can be a date string, or text like today, yesterday, tomorrow, Mon/Monday, etc.",
                required = False,
                #default = u""
                )

    actionitem_duedate = schema.TextLine(
                title = u"Due Date",
                description=u"When is this action due? Date can be a date string, or text like today, yesterday, tomorrow, Mon/Monday, etc.",
                required = False,
                #default = u""
                )
                
    actionitem_flag = schema.Bool(
            title = u"Flag",
            description=u"Flag can be used to temporarily highlight specific Actions. Currently, flagged actions show up at the top of the Active ActionItem portlet.",
            default=False)

class IActionItemListAdd(Interface):
    """Interface for ActionItemListAdd browser view
    """

    def setUpWidgets():
        """ Create widgets and fill with default values
        """

    def handle_add_action(action, data):
        """ Create a new NavigationItem in the proper place based on
        navigation_location_top_level.  Giving it a title and summary.
        """

    def handle_add_action_failure(action, data, errors):
        """"""

    def handle_add_action_validator(action, data):
        """ Validate fields during an Add action

            field data comes in from self.request

            (I think data is just for extra data,
                not the field data)
        """

class IActionItemListAddFields(Interface):
    """ Fields in the ActionItemListAdd form """

    actionitem_title = schema.TextLine(
                title = u"Action Name",
                description=u"What is the name of the action you want to add?",
                required = True,
                #default = u""
                )

    actionitem_context = schema.Choice(
                title = u"Context Tag",
                description = u"""Please choose a Context tag. Optional. A context is a physical or virtual location where you would perform this task (e.g. office, phone, staff meeting).""",
                required = False,   # note: false automatically generates a (no value) option
                vocabulary = 'ContextVocabulary',   # Vocabulary Factory
                )

    actionitem_newcontext = schema.TextLine(
                title = u"New Context Tag",
                description=u"If your Context doesn't exist in the above drop-down, type one here.",
                required = False,
                )

    actionitem_project = schema.Choice(
                title = u"Project Tag",
                description = u"""Please choose a Project tag. Optional.""",
                required = False,   # note: false automatically generates a (no value) option
                vocabulary = 'ProjectVocabulary',   # Vocabulary Factory
                )

    actionitem_newproject = schema.TextLine(
                title = u"New Project Tag",
                description=u"If your Project doesn't exist in the above drop-down, type one here.",
                required = False,
                )

    actionitem_startdate = schema.TextLine(
                title = u"Start Date",
                description=u"When would you like to see this action become active? Date can be a date string, or text like today, yesterday, tomorrow, Mon/Monday, etc.",
                required = False,
                #default = u""
                )

    actionitem_duedate = schema.TextLine(
                title = u"Due Date",
                description=u"When is this action due? Date can be a date string, or text like today, yesterday, tomorrow, Mon/Monday, etc.",
                required = False,
                #default = u""
                )

    actionitem_flag = schema.Bool(
            title = u"Flag",
            description=u"Flag can be used to temporarily highlight specific Actions. Currently, flagged actions show up at the top of the Active ActionItem portlet.",
            default=False)

class IActionItemListUpdate(Interface):
    """Interface for ActionItemListUpdate browser view
    """

class ICollectiveGTDUtility(Interface):
    """ A utility that returns site-wide information such as
        all Projects and Contexts
    """
    
    def getProjects(context):
        """ Return all Projects with an id/title tuple """

    def getContexts(context):
        """ Return all Contexts with an id/title tuple """
    
class IThemeSpecific(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 skin layer bound to a Skin
       Selection in portal_skins.
    """