# Test for Effort
# Author: Derek Hoshiko
# Created: 28 Mar 2008
# Last updated: 28/03/08


# Plone specific:
from Products.CollectiveGTD.tests import base
from Products.CollectiveGTD.modules.effort import Effort

class BasicEffort(base.CollectiveGTDTestCase):
    def testConstructor(self):
        effort = Effort()
        errStr = "Basic effort object could not be constructed."
        self.failUnless(effort.__class__.__name__ == 'Effort', errStr)

    def testBuildAndDisplayEffort(self):
        effort = Effort()
        errStr = "Attempt to construct '%s' effort object failed."
        listTests = [
            '(8-10)',
            '(2)',
            '(3+)',
            '(0-1)',
            '(0)',
            '(?)',
            '(10?)',
            '(1-3?)',
            '(1?)',
            '(1-2+)',
            '(+)',            
        ]
        for control in listTests:
            effort.Set(control)
            result = effort.DisplayEffort()
            self.failUnless(result == control, errStr % control)

    def testBadEffort(self):
        effort = Effort()
        errStr = "Bad construct '%s' not handled properly."
        listTests = [
            '8-10)',
            '()',
            '(?3)',
            '(0-1',
            '(s-r)',
            '(-?)',
            '(10?+)',
            '(1x3?)',
            '(p?)',
            '(1+2+)',
            '(-)',
            'Hello world.',
        ]
        for control in listTests:
            effort.Set(control)
            result = effort.DisplayEffort()
            self.failIf(result, errStr % control)

if __name__ == '__main__':
    unittest.main()
    
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(BasicEffort))
    return suite

