# Integration tests specific to collectivegtd
#

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CollectiveGTD.tests import base
from Products.CMFCore.utils import getToolByName
from Products.CollectiveGTD.config import ALLTYPES, ALLSKINS

class TestProductInstallation(base.CollectiveGTDTestCase):
    """ Ensure that our News Service product installs and 
        correctly configures our portal.
    """
    def afterSetUp(self):
        self.properties = self.portal.portal_properties
        self.factory    = self.portal.portal_factory
        self.skins      = self.portal.portal_skins
        self.types      = self.portal.portal_types
        self.workflow   = self.portal.portal_workflow
        self.catalog    = self.portal.portal_catalog
        
        self.collectivegtdTypes  = ALLTYPES
        self.collectivegtdTypeSkins  = ALLSKINS
    
    def testCoreProductsInstalled(self):
        CORE_PRODUCTS = ['CollectiveGTD',]
        
        for prod in CORE_PRODUCTS:
            self.failUnless(self.portal.portal_quickinstaller.isProductInstalled(prod),
                "Core product %s is not already installed" % prod)
    
    def testPortalFactorySetup(self):
        for t in self.collectivegtdTypes:
            self.failUnless(t in self.factory.getFactoryTypes(),
                "Type %s is not a factory types" % t)
    
    def testTypesNotListed(self):
        metaTypesNotToList  = self.properties.navtree_properties.getProperty('metaTypesNotToList')
        for t in self.collectivegtdTypes:
            self.failUnless(t in metaTypesNotToList,
                "Type %s will show up in the nav and shouldn't" % t)
    
    #def testTypesFTI(self):
    #    """Ensure FTI setting match settings in ZMI's portal_types tool.
    #    """
    #    FTIList = [
    #        {"ActionItemList":{
    #                "title":"Action Item List",
    #                "content_meta_type":"ActionItemList",
    #                "product":"collectivegtd",
    #                "global_allow":False,
    #                "filter_content_types":True,
    #                "allowed_content_types":('ActionItem',),
    #                "content_icon":'folder_icon.gif',
    #            },},
    #        
    #        {"ActionItem":{
    #                "title":"Action Item",
    #                "content_meta_type":"ActionItem",
    #                "product":"collectivegtd",
    #                "global_allow":False,
    #                "filter_content_types":False,
    #                "allowed_content_types":(),
    #                "content_icon":'document_icon.gif',
    #            },},
    #        
    #    ]
    #    for fti_type in FTIList:
    #        for k,v in fti_type.items():
    #            type_obj = getattr(self.types,k)
    #            for type_attr, type_attr_value in v.items():
    #                self.assertEquals(eval("type_obj.%s" % type_attr), type_attr_value,
    #                    "For type %s, the FTI value should be %s for the %s field \
    #                    types tool, but it's %s" % (k, type_attr_value, type_attr, eval("type_obj.%s" % type_attr)))

    def testFSDVInSkinsTool(self):
        """Test the the needed file system directory views have
           been registered with the skins tool.
        """
        for prodSkin in self.collectivegtdTypeSkins:
            self.failUnless(prodSkin in self.skins.objectIds(),
                "The skin %s was not registered with the skins tool" % prodSkin)
    
    def testSkinShowsUpInToolsLayers(self):
        """We need our product's skin directories to show up below custom as one of the called
           upon layers of our skin's properties
        """
        for selection, layers in self.skins.getSkinPaths():
            for specific_layer in self.collectivegtdTypeSkins:
                self.failUnless(specific_layer in layers, "The %s layer \
                    does not appear in the layers of Plone's %s skin" % (specific_layer,selection))
    
    def testCorePloneLayersStillPresent(self):
        """As was known to happen with legacy versions of GS working with the skins tool,
           it was easy to inadvertently remove need layers for the selected skin.  Here 
           we make sure this hasn't happened.
        """
        core_layers = ('plone_styles','plone_templates',)
        
        for selection, layers in self.skins.getSkinPaths():
            for specific_layer in core_layers:
                self.failUnless(specific_layer in layers, "The %s layer \
                    does not appear in the layers of Plone's available skins" % specific_layer)

    def testPortalActions(self):
        """ Test our new action(s) have been added """
        actionsTool = getToolByName(self.portal, 'portal_actions', None)
        all_actions = [action.getId() for action in actionsTool.listActions()]

    def testCatalogHasFieldsInMetadataColumns(self):
        """Make sure our custom indices exist
        """
        for field in ('ActionItemStartDate', 'ActionItemDueDate', \
            'ActionItemFlag', 'ActionItemComplete', 'ActionItemTags'):
            self.failUnless(field in self.catalog._catalog.schema.keys())

    def testCatalogHasFieldsInCatalogIndexes(self):
        for field in ('ActionItemStartDate', 'ActionItemDueDate', \
            'ActionItemFlag', 'ActionItemComplete', 'ActionItemTags'):
            self.failUnless(field in self.catalog.indexes())


if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductInstallation))
    return suite
