# Integration tests
#

__author__ = """Brian Gershon <briang@webcollective.coop>"""
__docformat__ = 'plaintext'

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CollectiveGTD.tests import base
from Products.CMFCore.utils import getToolByName

from zope.interface.verify import verifyObject, verifyClass

from Products.CollectiveGTD.interfaces import IActionItem, IActionItemList, \
    IActionItemListView, IActionItemListAdd, IActionItemListUpdate, \
    IActionItemListManager, IActionItemListAdd, IActionItemListImporter

from Products.CollectiveGTD.config import ALLTYPES
from Products.CollectiveGTD.content.ActionItem import ActionItem
from Products.CollectiveGTD.content.ActionItemList import ActionItemList
from Products.CollectiveGTD.browser.browser import ActionItemListView
from Products.CollectiveGTD.browser.browser import ActionItemListQuickAdd
from Products.CollectiveGTD.browser.browser import ActionItemListUpdate
from Products.CollectiveGTD.browser.browser import ActionItemListManager
from Products.CollectiveGTD.browser.browser import ActionItemListAdd
from Products.CollectiveGTD.browser.browser import ActionItemListImporter

class TestProductInterfaces(base.CollectiveGTDTestCase):
    """ Ensure that our product classes and objects
        fullfill their contractual interfaces
    """
    def afterSetUp(self):
        pass
    
    def testContentInterfaces(self):
        """ Some basic boiler plate testing of Interfaces and objects"""
        self.failUnless(IActionItem.implementedBy(ActionItem))
        self.failUnless(verifyClass(IActionItem, ActionItem))
    
        self.failUnless(IActionItemList.implementedBy(ActionItemList))
        self.failUnless(verifyClass(IActionItemList, ActionItemList))

    def testObjectInstances(self):
        # setup our content objects
        self.setupAllTypes()
        # verify we're an object of the expected class
        self.failUnless(isinstance(self.folder.actionitem, ActionItem))
        self.failUnless(isinstance(self.folder.actionitemlist, ActionItemList))
        
    def testContentObjectsVerify(self):
        # setup our content objects
        self.setupAllTypes()
        self.failUnless(verifyObject(IActionItem, self.folder.actionitem))
        self.failUnless(verifyObject(IActionItemList, self.folder.actionitemlist))
    
    def testBrowserViewInterfaces(self):
        self.failUnless(IActionItemListView.implementedBy(ActionItemListView))
        self.failUnless(verifyClass(IActionItemListView, ActionItemListView))

        self.failUnless(IActionItemListAdd.implementedBy(ActionItemListQuickAdd))
        self.failUnless(verifyClass(IActionItemListAdd, ActionItemListQuickAdd))

        self.failUnless(IActionItemListAdd.implementedBy(ActionItemListAdd))
        self.failUnless(verifyClass(IActionItemListAdd, ActionItemListAdd))

        self.failUnless(IActionItemListUpdate.implementedBy(ActionItemListUpdate))
        self.failUnless(verifyClass(IActionItemListUpdate, ActionItemListUpdate))

        self.failUnless(IActionItemListImporter.implementedBy(ActionItemListImporter))
        self.failUnless(verifyClass(IActionItemListImporter, ActionItemListImporter))

    def testBrowserViewsVerify(self):
        # get our view and verify we're an object of the expected class
        self.setupAllTypes()

        view = self.folder.actionitemlist.restrictedTraverse('@@actionitemlist_view')
        self.failUnless(isinstance(view, ActionItemListView))
        self.failUnless(verifyObject(IActionItemListView, view))

        view = self.folder.actionitemlist.restrictedTraverse('@@actionitemlist_quickadd')
        self.failUnless(isinstance(view, ActionItemListQuickAdd))
        self.failUnless(verifyObject(IActionItemListAdd, view))

        view = self.folder.actionitemlist.restrictedTraverse('@@actionitemlist_update')
        self.failUnless(isinstance(view, ActionItemListUpdate))
        self.failUnless(verifyObject(IActionItemListUpdate, view))

        view = self.folder.actionitemlist.restrictedTraverse('@@actionitemlist_importer')
        self.failUnless(isinstance(view, ActionItemListImporter))
        self.failUnless(verifyObject(IActionItemListImporter, view))

    def testAdapterInterfaces(self):
        self.failUnless(IActionItemListManager.implementedBy(ActionItemListManager))
        self.failUnless(verifyClass(IActionItemListManager, ActionItemListManager))

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestProductInterfaces))
    return suite
