#
# Test ActionItemListKSS
#

__author__ = """Brian Gershon <briang@webcollective.coop>"""
__docformat__ = 'plaintext'

import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFCore.utils import getToolByName
from DateTime import DateTime

from Products.CollectiveGTD.tests import base

# zope
from zope.interface.verify import verifyClass
from DateTime import DateTime
from AccessControl import Unauthorized
from Products.CMFPlone.utils import transaction

# content objects
from Products.CollectiveGTD.content import ActionItem, ActionItemList

from zope.publisher.browser import TestRequest
from zope.component import getMultiAdapter

class TestActionItemListKSS(base.CollectiveGTDTestCase):
    """ Ensure that ActionItemListKSS is working correctly.
    """

    def afterSetUp(self):
        self.workflow_tool = getToolByName(self.portal, 'portal_workflow')
        self.portal_catalog = getToolByName(self.portal, 'portal_catalog')
        self.portal.acl_users._doAddUser('manager', 'secret', ['Member', 'Manager'], [])
        
        # grab actionitemlist created in base.CollectiveGTDTestCase
        self.setupAllTypes()
        self.actionitemlist = self.folder.restrictedTraverse('actionitemlist')
    
    def testUpdateDueDate(self):
        """ Test DueDate field is set correctly.
        """
        request = TestRequest()
        request.RESPONSE = request.response

        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one')
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two')
        
        actionitem_one =  self.actionitemlist.restrictedTraverse('my-actionitem-one')
        actionitem_two =  self.actionitemlist.restrictedTraverse('my-actionitem-two')

        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_kss_update_duedate')

        # try normal date
        dt = '3/1/2008'
        view.update_dueDate('my-actionitem-one', dt)
        self.failUnless(actionitem_one.ActionItemDueDate().Date() == DateTime(dt).Date(), 'bad date')

        # try datetimelingo
        view.update_dueDate('my-actionitem-one', 'today')
        self.failUnless(actionitem_one.ActionItemDueDate().Date() == DateTime().Date(), 'bad date')
        
        # try clearing date
        view.update_dueDate('my-actionitem-one', '')
        self.failUnless(actionitem_one.ActionItemDueDate() is None, 'bad date')

    def testUpdateStartDate(self):
        """ Test StartDate field is set correctly.
        """
        request = TestRequest()
        request.RESPONSE = request.response

        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one')
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two')

        actionitem_one =  self.actionitemlist.restrictedTraverse('my-actionitem-one')
        actionitem_two =  self.actionitemlist.restrictedTraverse('my-actionitem-two')

        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_kss_update_startdate')

        # try normal date
        dt = '3/1/2008'
        view.update_startDate('my-actionitem-one', dt)
        self.failUnless(actionitem_one.ActionItemStartDate().Date() == DateTime(dt).Date(), 'bad date')

        # try datetimelingo
        view.update_startDate('my-actionitem-one', 'today')
        self.failUnless(actionitem_one.ActionItemStartDate().Date() == DateTime().Date(), 'bad date')

        # try clearing date
        view.update_startDate('my-actionitem-one', '')
        self.failUnless(actionitem_one.ActionItemStartDate() is None, 'bad date')

    def testUpdateFlag(self):
        """ Test Flag field is set correctly.
        """
        request = TestRequest()
        request.RESPONSE = request.response

        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one')
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two')

        actionitem_one =  self.actionitemlist.restrictedTraverse('my-actionitem-one')
        actionitem_two =  self.actionitemlist.restrictedTraverse('my-actionitem-two')

        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_kss_update_flag')

        flag = 'on'
        view.update_flag('my-actionitem-one', flag)
        self.failUnless(actionitem_one.ActionItemFlag(), 'bad flag')

        flag = 'off'
        view.update_flag('my-actionitem-one', flag)
        self.failUnless(not actionitem_one.ActionItemFlag(), 'bad flag')

    def testUpdateComplete(self):
        """ Test Complete field is set correctly.
        """
        request = TestRequest()
        request.RESPONSE = request.response

        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one')
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two')

        actionitem_one =  self.actionitemlist.restrictedTraverse('my-actionitem-one')
        actionitem_two =  self.actionitemlist.restrictedTraverse('my-actionitem-two')

        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_kss_update_complete')

        complete = 'on'
        view.update_complete('my-actionitem-one', complete)
        self.failUnless(actionitem_one.ActionItemComplete(), 'bad complete')

        complete = 'off'
        view.update_complete('my-actionitem-one', complete)
        self.failUnless(not actionitem_one.ActionItemComplete(), 'bad complete')

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestActionItemListKSS))
    return suite
