#
# Test ActionItemQuickAdd functionality
#

__author__ = """Brian Gershon <briang@webcollective.coop>"""
__docformat__ = 'plaintext'

import os, sys, email

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFCore.utils import getToolByName
from DateTime import DateTime

from Products.CollectiveGTD.tests import base

# zope
from zope.interface.verify import verifyClass
from DateTime import DateTime
from AccessControl import Unauthorized
from Products.CMFPlone.utils import transaction

# interfaces
#from Products.CollectiveGTD.interfaces import INavigationFolder, INavigationItem, INavigationPortletView
#from Products.ATContentTypes.interfaces import IATFolder, IATNewsItem

# content objects
from Products.CollectiveGTD.content import ActionItem, ActionItemList

from zope.publisher.browser import TestRequest
from zope.component import getMultiAdapter

class TestQuickAdd(base.CollectiveGTDTestCase):
    """ Ensure that ActionItemList QuickAdd is working correctly.
    """

    def afterSetUp(self):
        self.workflow_tool = getToolByName(self.portal, 'portal_workflow')
        self.portal_catalog = getToolByName(self.portal, 'portal_catalog')
        self.portal.acl_users._doAddUser('manager', 'secret', ['Member', 'Manager'], [])
        
        # grab actionitemlist created in base.CollectiveGTDTestCase
        self.setupAllTypes()
        self.actionitemlist = self.folder.restrictedTraverse('actionitemlist')
    
    def testActionItemDictionaryFields(self):
        """ Test fields are set correctly, and that dictionary of fields
            are properly returned from 'actionitemlist_view'
        """
        request = TestRequest()
        request.RESPONSE = request.response

        # add some test content
        # TODO: Create a view for updating ActionItems (instead of directly invoking?)
        #       or use the QuickAdd form.
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                title="My ActionItem One",
                startDate="1/1/07",
                dueDate="5/1/07",
                tags=['context:home','project:Acme Plone Widget',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                title="My ActionItem Two",
                startDate="2/1/07",
                dueDate="6/1/07",
                tags=['context:office','project:Clean the office',],
                )
        
        actionitem_one =  self.actionitemlist.restrictedTraverse('my-actionitem-one')
        actionitem_two =  self.actionitemlist.restrictedTraverse('my-actionitem-two')

        self.logout()

        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_view')
        
        getActionItems = view.getSortedActions(sortview='all', \
                sortlimit='all', filter_by_tag='')

        self.failUnless(len(getActionItems) == 2, \
            'There should be 2 returned dictionaries with ActionItem details in them')
        
        for i in getActionItems:
            if i['id'] == 'my-actionitem-one':
                actionitem_one_dict = i
            if i['id'] == 'my-actionitem-two':
                actionitem_two_dict = i

        self.failUnless(actionitem_one_dict['id'] == 'my-actionitem-one', 'bad value')
        self.failUnless(actionitem_one_dict['title'] == "My ActionItem One", 'bad value')
        self.failUnless(actionitem_one_dict['startDate'] == DateTime('2007/01/01'), 'bad value')
        self.failUnless(actionitem_one_dict['dueDate'] == DateTime('2007/05/01'), 'bad value')
        self.failUnless(actionitem_one_dict['startDateDisplay']=='2007/01/01', \
            'expected European date formatting which Quick view depends on for sorting')
        self.failUnless(actionitem_one_dict['dueDateDisplay']=='2007/05/01', \
            'expected European date formatting which Quick view depends on for sorting')
        self.failUnless(actionitem_one_dict['absolute_url'] == 'http://nohost/plone/Members/test_user_1_/actionitemlist/my-actionitem-one', 'bad value')
        self.failUnless(not actionitem_one_dict['complete'], 'bad value')
        self.failUnless(actionitem_one_dict['priority'] == '(not implemented yet)', 'bad value')
        self.failUnless(not actionitem_one_dict['flag'], 'bad value')
        self.failUnless(actionitem_one_dict['recurring'] == '(not implemented yet)', 'bad value')
        self.failUnless(actionitem_one_dict['effort'] == '(not implemented yet)', 'bad value')
        self.failUnless(actionitem_one_dict['portal_type'] == 'ActionItem', 'bad value')
        self.failUnless(actionitem_one_dict['description'] == '', 'bad value')
        self.failUnless(actionitem_one_dict.has_key('created'), 'missing key')
        self.failUnless('home' in actionitem_one_dict['contexts'], 'bad value')
        self.failUnless('Acme Plone Widget' in actionitem_one_dict['projects'], 'bad value')

    def testViewLogicForReturningUpcomingActions(self):
        """ Test our view logic. Are we returning actions that are properly
            ordered and filtered?
        
        Should return:
            RESULT SET:
                Only non-completed items
                No actions with a startDate into the future
                    (Not: None, In the past, or Today)

            SORT BY:
                All flagged items first
                All items with dueDates (sorted by dueDate)
                All remaining items without dueDates and not flagged
        """
        request = TestRequest()
        request.RESPONSE = request.response

        # create some popular date strings
        now = DateTime()
        today = DateTime.Date(now)
        yesterday = DateTime.Date(now - 1)
        tomorrow = DateTime.Date(now + 1)
        following_day = DateTime.Date(now + 2)
        far_past = DateTime.Date(now - 100)
        far_future = DateTime.Date(now + 100)

        #
        # add some test content
        #

        # this item should appear first because it's flagged
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-FLAGGED',
                title="My ActionItem FLAGGED",
                flag = True,
                )

        # test startDate of yesterday
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                title="My ActionItem One",
                startDate=yesterday,
                dueDate=following_day,
                )

        # will not appear because startDate is tomorrow
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                title="My ActionItem Two",
                startDate=tomorrow,
                dueDate=tomorrow,
                )

        # test startDate of today
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-three',
                title="My ActionItem Three",
                startDate=today,
                dueDate=far_future,
                )

        # test same startDate and dueDate of today
        # 'four' should come before 'three' because of dueDate
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-four',
                title="My ActionItem Four",
                startDate=today,
                dueDate=today,
                )

        # should appear because startDate has already passed. dueDate is in the future too.
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-five',
                title="My ActionItem Five",
                startDate=far_past,
                dueDate=yesterday,
                )

        # should appear at the end because no startDate and no dueDate
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-NODATES',
                title="My ActionItem Five",
                )

        # this item should not appear as upcoming since it's complete!
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-DONE',
                title="My ActionItem DONE",
                complete = True,
                )

        # this item should not appear as upcoming since startDate is in the future!
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-STARTS-IN-FUTURE',
                title="My ActionItem DONE",
                startDate=far_future,
                )

        self.logout()
        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_view')

        actions = view.getSortedActions(sortview='upcoming', \
                sortlimit='all', filter_by_tag='')

        # TEST RESULT SET
        #print "debug", [(a['id'],a['flag'],a['startDate'],a['dueDate']) for a in actions]
        
        self.failUnless(len(actions) == 6, \
            'There should be 6 returned dictionaries with ActionItem details in them' + \
                ' but instead these were returned: %s' % [a['id'] for a in actions])

        #actionitem_one =  self.actionitemlist.restrictedTraverse('my-actionitem-one')
        #actionitem_two =  self.actionitemlist.restrictedTraverse('my-actionitem-two')

        #for i in actions:
        #    if i['id'] == 'my-actionitem-one':
        #        actionitem_one_dict = i
        #    if i['id'] == 'my-actionitem-two':
        #        actionitem_two_dict = i

        # TEST SORT ORDER
        self.failUnless(actions[0]['id'] == 'my-actionitem-FLAGGED', "wrong order")
        self.failUnless(actions[1]['id'] == 'my-actionitem-five', "wrong order")
        self.failUnless(actions[2]['id'] == 'my-actionitem-four', "wrong order")
        self.failUnless(actions[3]['id'] == 'my-actionitem-one', "wrong order")
        self.failUnless(actions[4]['id'] == 'my-actionitem-three', "wrong order")
        self.failUnless(actions[5]['id'] == 'my-actionitem-NODATES', "wrong order")

        # these should NOT appear:
        
        exists = [a for a in actions if a['id'] == 'my-actionitem-DONE']
        self.failUnless(not exists, \
            "my-actionitem-DONE should not appear in since it is marked 'complete'")
        
        exists = [a for a in actions if a['id'] == 'my-actionitem-STARTS-IN-FUTURE']
        self.failUnless(not exists, \
            "my-actionitem-STARTS-IN-FUTURE should not appear in since it starts in the future")
        
        # TEST VALUES
        #self.failUnless(actionitem_one_dict['id'] == 'my-actionitem-one', 'bad value')
        #self.failUnless(actionitem_one_dict['startDate'] == DateTime('2008/01/01'), 'bad value')
        #self.failUnless(actionitem_one_dict['dueDate'] == DateTime('2008/05/01'), 'bad value')
        #self.failUnless(not actionitem_one_dict['complete'], 'bad value')
        #self.failUnless(actionitem_one_dict['priority'] == '', 'bad value')
        #self.failUnless(not actionitem_one_dict['flag'], 'bad value')

        #self.failUnless(actionitem_two_dict['id'] == 'my-actionitem-two', 'bad value')
        #self.failUnless(actionitem_two_dict['startDate'] == DateTime('2008/02/01'), 'bad value')
        #self.failUnless(actionitem_two_dict['dueDate'] == DateTime('2008/06/01'), 'bad value')
        #self.failUnless(not actionitem_two_dict['complete'], 'bad value')
        #self.failUnless(actionitem_two_dict['priority'] == '', 'bad value')
        #self.failUnless(not actionitem_two_dict['flag'], 'bad value')

    def testCSS_Styles(self):
        """
        Items due tomorrow, today or past-due should "due" (e.g. red).
        Items due in two days should be "upcoming" (e.g. light red).
        Other items should have no css_class.
        """

        request = TestRequest()
        request.RESPONSE = request.response

        # create some popular date strings
        now = DateTime()
        far_past = DateTime.Date(now - 100)
        yesterday = DateTime.Date(now - 1)
        today = DateTime.Date(now)
        tomorrow = DateTime.Date(now + 1)
        following_day = DateTime.Date(now + 2)
        far_future = DateTime.Date(now + 100)
        
        # should appear because startDate has already passed. dueDate is in the future too.
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                dueDate=far_past,
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                dueDate=yesterday,
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-three',
                dueDate=today,
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-four',
                dueDate=tomorrow,
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-five',
                dueDate=following_day,
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-six',
                dueDate=far_future,
                )
        
        self.logout()
        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_view')
        actions = view.getSortedActions(sortview='all', \
                sortlimit='all', filter_by_tag='')

        self.failUnless(actions[0]['id'] == 'my-actionitem-one' and \
            actions[0]['css_class'] == 'action-due', 'action should have "action-due" style')
        self.failUnless(actions[1]['id'] == 'my-actionitem-two' and \
            actions[1]['css_class'] == 'action-due', 'action should have "action-due" style')
        self.failUnless(actions[2]['id'] == 'my-actionitem-three' and \
            actions[2]['css_class'] == 'action-due', 'action should have "action-due" style')
        self.failUnless(actions[3]['id'] == 'my-actionitem-four' and \
            actions[3]['css_class'] == 'action-due', 'action should have "action-due" style')
        self.failUnless(actions[4]['id'] == 'my-actionitem-five' and \
            actions[4]['css_class'] == 'action-upcoming', 'action should have "action-upcoming" style')
        self.failUnless(actions[5]['id'] == 'my-actionitem-six' and \
            actions[5]['css_class'] == '', 'This action should have "" style')

    def testGetAllProjectsAndContexts(self):
        """ Test getAllProjectsAndContexts() """

        request = TestRequest()
        request.RESPONSE = request.response

        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                title="My ActionItem One",
                startDate="1/1/07",
                dueDate="5/1/07",
                tags=['context:home','project:Acme Plone Widget',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                title="My ActionItem Two",
                startDate="5/1/07",
                dueDate="4/1/08",
                tags=['context:office','project:Clean the office',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-three',
                title="My ActionItem Three",
                startDate="1/1/06",
                dueDate="6/1/06",
                tags=['context:office','project:Clean the office',],
                )
        
        self.logout()
        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_view')
        
        projectsAndContexts = view.getAllProjectsAndContexts()
        
        # should return 4 projects, grouped by contexts then projects, sorted
        # by name.
        self.failUnless(len(projectsAndContexts) == 4, 'Not all Contexts/Projects were returned')
        self.failUnless(projectsAndContexts[0][0] == 'context:home', 'bad value')
        self.failUnless(projectsAndContexts[1][0] == 'context:office', 'bad value')
        self.failUnless(projectsAndContexts[2][0] == 'project:acme-plone-widget', 'bad value')
        self.failUnless(projectsAndContexts[3][0] == 'project:clean-the-office', 'bad value')
    
    def testGetSortedActions(self):
        """
        Test the various sort options, particularly re: DueDate and StartDate
        """
        request = TestRequest()
        request.RESPONSE = request.response

        # add some test content
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                title="My ActionItem One",
                startDate="1/1/07",
                dueDate="5/1/07",
                tags=['context:home','project:Acme Plone Widget',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                title="My ActionItem Two",
                startDate="5/1/07",
                dueDate="4/1/08",
                tags=['context:office','project:Clean the office',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-three',
                title="My ActionItem Three",
                startDate="1/1/06",
                dueDate="6/1/06",
                tags=['context:office','project:Clean the office',],
                )

        self.logout()
        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_view')

        getActionItems = view.getSortedActions(sortview='sort_startdate', \
                sortlimit='all', filter_by_tag='')

        self.failUnless(len(getActionItems) == 3, \
            'There should be 3 returned dictionaries with ActionItem details in them')
        self.failUnless(getActionItems[0]['id'] == 'my-actionitem-two', 'out of order')
        self.failUnless(getActionItems[1]['id'] == 'my-actionitem-one', 'out of order')
        self.failUnless(getActionItems[2]['id'] == 'my-actionitem-three', 'out of order')

        getActionItems = view.getSortedActions(sortview='sort_duedate', \
                sortlimit='all', filter_by_tag='')

        self.failUnless(len(getActionItems) == 3, \
            'There should be 3 returned dictionaries with ActionItem details in them')
        self.failUnless(getActionItems[0]['id'] == 'my-actionitem-two', 'out of order')
        self.failUnless(getActionItems[1]['id'] == 'my-actionitem-one', 'out of order')
        self.failUnless(getActionItems[2]['id'] == 'my-actionitem-three', 'out of order')

        # test filter_by_tag functionality
        # ... for underlying _getActionItems() family of searches
        getActionItems = view.getSortedActions(sortview='sort_duedate', \
                sortlimit='all', filter_by_tag='context:office')
        self.failUnless(len(getActionItems) == 2, \
            'There should be 2 ActionItems (filter by "context:office").')

        getActionItems = view.getSortedActions(sortview='sort_duedate', \
                sortlimit='all', filter_by_tag='project:acme-plone-widget')
        self.failUnless(len(getActionItems) == 1, \
            'There should be 1 ActionItem (filter by "project:Acme Plone Widget").')

        # test filter_by_tag functionality
        # ... for underlying _getUpcomingActions() search
        getActionItems = view.getSortedActions(sortview='upcoming', \
                sortlimit='all', filter_by_tag='context:office')
        self.failUnless(len(getActionItems) == 2, \
            'There should be 2 ActionItems (filter by "context:office").')
            
        # test filter_by_tag functionality using "all" keyword
        getActionItems = view.getSortedActions(sortview='sort_duedate', \
                sortlimit='all', filter_by_tag='all')
        self.failUnless(len(getActionItems) == 3, \
            'There should be 3 ActionItems (filter turned off).')        

        getActionItems = view.getSortedActions(sortview='upcoming', \
                sortlimit='all', filter_by_tag='all')
        self.failUnless(len(getActionItems) == 3, \
            'There should be 3 ActionItems (filter turned off).')

    def testGetRecentlyAddedActions(self):
        """ Test fields are set correctly, and that dictionary of fields
            are properly returned from 'actionitemlist_view'
        """
        request = TestRequest()
        request.RESPONSE = request.response

        # add some test content
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                title="My ActionItem One",
                startDate="1/1/07",
                dueDate="5/1/07",
                tags=['context:home','project:Acme Plone Widget',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                title="My ActionItem Two",
                startDate="2/1/07",
                dueDate="6/1/07",
                tags=['context:office','project:Clean the office',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-three',
                title="My ActionItem Three",
                startDate="1/1/06",
                dueDate="6/1/06",
                tags=['context:office','project:Clean the office',],
                )

        self.logout()
        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_view')
        getActionItems = view.getSortedActions(sortview='recently_added', \
                sortlimit='all', filter_by_tag='')

        self.failUnless(len(getActionItems) == 3, \
            'There should be 3 returned dictionaries with ActionItem details in them')
        self.failUnless(getActionItems[0]['id'] == 'my-actionitem-three', 'out of order')
        self.failUnless(getActionItems[1]['id'] == 'my-actionitem-two', 'out of order')
        self.failUnless(getActionItems[2]['id'] == 'my-actionitem-one', 'out of order')

        # test filtering
        getActionItems = view.getSortedActions(sortview='recently_added', \
                sortlimit='all', filter_by_tag='context:office')
        self.failUnless(len(getActionItems) == 2, \
            "There should be 2 returned dictionaries since we're filtering by 'context:office'")

    def testGetAllActiveActions_XML(self):
        """ Test the getAllActiveActions(outputFormat="xml") method. """
        request = TestRequest()
        request.RESPONSE = request.response

        # add some test content
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                title="My ActionItem One",
                startDate="1/1/07",
                dueDate="5/1/07",
                tags=['context:home','project:Acme Plone Widget',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                title="My ActionItem Two",
                startDate="2/1/07",
                dueDate="6/1/07",
                tags=['context:office','project:Clean the office',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-three',
                title="My ActionItem Three",
                startDate="1/1/06",
                dueDate="6/1/06",
                tags=['context:office','project:Clean the office',],
                )
        # try one with no tags to make sure XML returns an empty element for
        #   both context and project.
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-four',
                title="My ActionItem Four",
                startDate="1/1/06",
                dueDate="9/1/08",
                )

        self.logout()
        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_view')
        xml = view.getAllActiveActions()
        
        results = """<collectivegtd language="en"><actionitems><actionitem duedate="06/01/2006" flag="False" id="my-actionitem-three" startdate="01/01/2006" title="My ActionItem Three"><project_tags><tag title="Clean the office" /></project_tags><context_tags><tag title="office" /></context_tags></actionitem><actionitem duedate="05/01/2007" flag="False" id="my-actionitem-one" startdate="01/01/2007" title="My ActionItem One"><project_tags><tag title="Acme Plone Widget" /></project_tags><context_tags><tag title="home" /></context_tags></actionitem><actionitem duedate="06/01/2007" flag="False" id="my-actionitem-two" startdate="02/01/2007" title="My ActionItem Two"><project_tags><tag title="Clean the office" /></project_tags><context_tags><tag title="office" /></context_tags></actionitem><actionitem duedate="09/01/2008" flag="False" id="my-actionitem-four" startdate="01/01/2006" title="My ActionItem Four"><project_tags /><context_tags /></actionitem></actionitems><allcontexts><context title="context:home" /><context title="context:office" /></allcontexts><allprojects><project title="project:acme-plone-widget" /><project title="project:clean-the-office" /></allprojects></collectivegtd>"""

        self.failUnless(xml == results, "XML output is not correct.  This is what was returned %s" % xml)
        
    def testActionItemQuickAddForm_RequiredFieldsOnly(self):
        """ Add a new ActionItem via the ActionItemListQuickAdd form.
        
            Testing defaults when only required fields are input.
        """
        
        # NOTE: Intentionally using None for startdate and duedate, since this
        #       is what formlib passes.  Want to make sure None doesn't get
        #       treated like a string and added as a real context or project tag.
        request = TestRequest(form={ 'form.actionitem_title':'A Test ActionItem',
                                     'form.actionitem_startdate': u'None',
                                     'form.actionitem_duedate': u'None',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        myform = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_quickadd')
        myform.update()

        # did the update work?
        actionitem_id = 'a-test-actionitem'
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of "%s"' % myform.status)
        self.failUnless(actionitem_id in self.actionitemlist.objectIds(), "A new 'a-test-actionitem' ActionItem was not created")

        # Does it have the right data?
        actionitem = self.actionitemlist.restrictedTraverse(actionitem_id)
        self.failUnless(actionitem.id == actionitem_id, "Incorrect ID")
        self.failUnless(actionitem.title == u'A Test ActionItem', "Incorrect Title")
        self.failUnless(actionitem.startDate is None, "Incorrect StartDate")
        self.failUnless(actionitem.dueDate is None, "Incorrect DueDate")
        # TODO: Note that I wasn't able to get this test to fail
        #       properly if the form passes None "Project" or "Context".
        #       Passing that same value in this produces different results.
        #       I tried passing None and u'None' but both pass this test
        #       even when form breaks.  I fixed the bug in handle_add_action()
        #       but can't test it.
        self.failUnless(len(actionitem.tags) == 0, "Incorrect number of tags")

    def testActionItemQuickAddForm(self):
        """ Add a new ActionItem via the ActionItemListQuickAdd form.
        """
        # create some Projects and Contexts
        # TODO: Create a view for updating ActionItems (instead of directly invoking?)
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                tags=['context:home','project:Acme Plone Widget',],
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                tags=['context:office','project:Clean the office',],
                )

        # NOTE: form.navigation_location_top_level must match a vocab Title or will err during .update()
        request = TestRequest(form={ 'form.actionitem_title':'A Test ActionItem',
                                     'form.actionitem_startdate': "1/1/08",
                                     'form.actionitem_duedate': "5/1/08",
                                     # TODO: we should pass in the context Title, yes?
                                     #   Right now id and title are the same.
                                     'form.actionitem_context': "context:office",
                                     # TODO: we should pass in the project Title, yes?
                                     #   Right now id and title are the same.
                                     'form.actionitem_project': "project:acme-plone-widget",
                                     'form.actionitem_flag': 'on', 
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        
        myform = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_quickadd')
        
        myform.update()

        # did the update work?
        actionitem_id = 'a-test-actionitem'
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of "%s"' % myform.status)
        self.failUnless(actionitem_id in self.actionitemlist.objectIds(), "A new 'a-test-actionitem' ActionItem was not created")
        
        # Does it have the right data?
        actionitem = self.actionitemlist.restrictedTraverse(actionitem_id)
        self.failUnless(actionitem.id == actionitem_id, "Incorrect ID")
        self.failUnless(actionitem.title == u'A Test ActionItem', "Incorrect Title")
        self.failUnless(actionitem.startDate == DateTime('2008/01/01'), "Incorrect StartDate")
        self.failUnless(actionitem.dueDate == DateTime('2008/05/01'), "Incorrect DueDate")
        self.failUnless(actionitem.flag, "Flag should have been True")

        self.failUnless(len(actionitem.tags) == 2, "Incorrect number of tags")
        self.failUnless('project:Acme Plone Widget' in actionitem.tags, "Incorrect project")
        self.failUnless('context:office' in actionitem.tags, "Incorrect context")

    def testActionItemQuickAddForm_NewProjectNewContext(self):
        """ Add a new ActionItem via the ActionItemListQuickAdd form, but
            we're adding new Context and new Project instead of using
            existing ones from drop-down.
        """

        # NOTE: form.navigation_location_top_level must match a vocab Title or will err during .update()
        request = TestRequest(form={ 'form.actionitem_title':'A Test ActionItem',
                                     'form.actionitem_newcontext': 'coffeeshop',
                                     'form.actionitem_newproject': 'Flower Picking',
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response
        myform = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_quickadd')
        myform.update()

        # did the update work?
        actionitem_id = 'a-test-actionitem'
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of "%s"' % myform.status)
        self.failUnless(actionitem_id in self.actionitemlist.objectIds(), "A new 'a-test-actionitem' ActionItem was not created")

        # Does it have the right data?
        actionitem = self.actionitemlist.restrictedTraverse(actionitem_id)
        self.failUnless(actionitem.id == actionitem_id, "Incorrect ID")
        self.failUnless(actionitem.title == u'A Test ActionItem', "Incorrect Title")
        self.failUnless(len(actionitem.tags) == 2, "%s is the wrong number of tags" % \
            len(actionitem.tags))
        self.failUnless('project:Flower Picking' in actionitem.tags, "Incorrect new project")
        self.failUnless('context:coffeeshop' in actionitem.tags, "Incorrect new context")

    def testActionItemQuickAddForm_BadDates(self):
        """ Add a new ActionItem via the ActionItemListQuickAdd form.
        """

        request = TestRequest(form={ 'form.actionitem_title':'A Test ActionItem',
                                     'form.actionitem_startdate': "bad startdate format",
                                     'form.actionitem_duedate': "bad duedate format",
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response

        myform = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_quickadd')

        myform.update()

        # did the update work?
        actionitem_id = 'a-test-actionitem'
        self.failUnless(myform.status != 'Updated successfully.', 'This form should have failed, but instead had form.status of %s' % myform.status)
        self.failUnless(actionitem_id not in self.actionitemlist.objectIds(), "A new 'a-test-actionitem' should not have been created.")

    def testActionItemQuickAddForm_WithUnicode(self):
        """ Make sure quick add works with unicode characters.
        """
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                tags=['context:é, è, ê','project:à, â',],
                )
        
        request = TestRequest(form={ 'form.actionitem_title':'é, è, ê, à, â, ô, ù',
                                     'form.actionitem_startdate': "1/1/08",
                                     'form.actionitem_duedate': "5/1/08",
                                     'form.actionitem_context': "context:e-e-e",
                                     'form.actionitem_project': "project:a-a",
                                     'form.actionitem_flag': 'on', 
                                     'form.actions.add':'Add'})
        request.RESPONSE = request.response

        myform = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_quickadd')

        myform.update()

        # did the update work?
        actionitem_id = 'e-e-e-a-a-o-u'
        self.failUnless(myform.status == 'Updated successfully.', 'Wrong form.status of "%s"' % myform.status)
        self.failUnless(actionitem_id in self.actionitemlist.objectIds(), "A new 'e-e-e-a-a-o-u' ActionItem was not created")

        # Does it have the right data?
        actionitem = self.actionitemlist.restrictedTraverse(actionitem_id)
        self.failUnless(actionitem.id == actionitem_id, "Incorrect ID")
        #self.failUnless(actionitem.title.decode('latin-1') == u'é, è, ê, à, â, ô, ù', "Incorrect Title")
        self.failUnless(actionitem.startDate == DateTime('2008/01/01'), "Incorrect StartDate")
        self.failUnless(actionitem.dueDate == DateTime('2008/05/01'), "Incorrect DueDate")
        self.failUnless(actionitem.flag, "Flag should have been True")

        self.failUnless(len(actionitem.tags) == 2, "Incorrect number of tags")
        self.failUnless('project:à, â'.decode('utf8') in actionitem.tags, "Incorrect project")
        self.failUnless('context:é, è, ê'.decode('utf8') in actionitem.tags, "Incorrect context")

    def testUpdate(self):
        """ Test our update view.
        """
        # add some test content
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-one',
                title="My ActionItem One",
                startDate="1/1/08",
                dueDate="5/1/08",
                )
        self.actionitemlist.invokeFactory('ActionItem', id='my-actionitem-two',
                title="My ActionItem Two",
                startDate="2/1/08",
                dueDate="6/1/08",
                )

        actionitem_one =  self.actionitemlist.restrictedTraverse('my-actionitem-one')
        actionitem_two =  self.actionitemlist.restrictedTraverse('my-actionitem-two')

        self.failUnless(not actionitem_one.ActionItemComplete(), "should default to False")
        self.failUnless(not actionitem_one.ActionItemFlag(), "should default to False")
        self.failUnless(not actionitem_two.ActionItemComplete(), "should default to False")
        self.failUnless(not actionitem_two.ActionItemFlag(), "should default to False")

        # set both fields to 'on'
        request = TestRequest(actionitems=[{ 'id':'my-actionitem-one', \
                                             'complete': 'on', \
                                             'flag': 'on'},
                                           { 'id':'my-actionitem-two', \
                                             'complete': 'on', \
                                             'flag': 'on'},
                                          ])
        request.RESPONSE = request.response
        updateform = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_update')
        updateform()
        self.failUnless(actionitem_one.ActionItemComplete(), "should have been set to True")
        self.failUnless(actionitem_one.ActionItemFlag(), "should have been set to True")
        self.failUnless(actionitem_two.ActionItemComplete(), "should have been set to True")
        self.failUnless(actionitem_two.ActionItemFlag(), "should have been set to True")

        # set both fields to False
        request = TestRequest(actionitems=[{ 'id':'my-actionitem-one'},
                                           { 'id':'my-actionitem-two'},
                                          ])
        request.RESPONSE = request.response
        updateform = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_update')
        updateform()
        self.failUnless(not actionitem_one.ActionItemComplete(), "should default to False")
        self.failUnless(not actionitem_one.ActionItemFlag(), "should default to False")
        self.failUnless(not actionitem_two.ActionItemComplete(), "should default to False")
        self.failUnless(not actionitem_two.ActionItemFlag(), "should default to False")

    def testDateTimeLingoDateDisplay(self):
        """
        Test the various dateDisplay() formats used in views.

        Some current barebones testing happening in the
        "testContentFieldsAndView" test to make sure the
        European date format is returned ('yyyy/mm/dd') for
        sorting.
        
        FIXME: Need to test strings like this:
        
            "2007/02/21 (today)"
            "2007/02/21 (tomorrow)"
            "2007/02/21 (Monday)"
        
        """
        #self.fail('test not implemented yet')
        pass

    def testImporter_Valid(self):
        """ Test Importer View with valid data.
        """
        # NOTE: form.navigation_location_top_level must match a vocab Title or 
        # will err during .update()
        formFields = { 
            'form.actionitem_title':'A Test ActionItem',
            'form.actionitem_startdate': "1/1/08",
            'form.actionitem_duedate': "5/1/08",
            'form.actionitem_newcontext': "office",
            'form.actionitem_newproject': "Acme Plone Widget",
            'form.actionitem_flag': 'on',
            'form.actionitem_notes': "A quick brown fox\njumps over the lazy dog.",
            'form.actions.add':'Add'
        }
        request = TestRequest(form=formFields)

        request.RESPONSE = request.response     # formlib needs this
        view = getMultiAdapter((self.actionitemlist, request), name='actionitemlist_importer')
        view()    # call view to import

        # did the update work?
        self.failUnless(request.response.getStatus() == 200, 'Request should have succeeded')

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestQuickAdd))
    return suite
